from __future__ import absolute_import
from __future__ import print_function
import re

# Object ID counter for object sorting key
global_object_counter = 0

operator_dict = {
    'Uminus': '-', 'Ulnot': '!', 'Unot': '~', 'Uand': '&', 'Unand': '~&',
    'Uor': '|', 'Unor': '~|', 'Uxor': '^', 'Uxnor': '~^',
    'Power': '**', 'Times': '*', 'Divide': '/', 'Mod': '%',
    'Plus': '+', 'Minus': '-',
    'Sll': '<<', 'Srl': '>>', 'Sra': '>>>',
    'LessThan': '<', 'GreaterThan': '>', 'LessEq': '<=', 'GreaterEq': '>=',
    'Eq': '==', 'NotEq': '!=', 'Eql': '===', 'NotEql': '!==',
    'And': '&', 'Xor': '^', 'Xnor': '~^',
    'Or': '|', 'Land': '&&', 'Lor': '||'
}


def op2mark(op):
    return operator_dict[op]


def str_to_signed(s):
    targ = s.replace('_', '')
    match = re.search(r's(.+)', targ)
    if match is not None:
        return True
    return False


def check_int_hex(v):
    if not re.search(r'^[0-9a-fA-FxzXZ]+$', v):
        raise ValueError("Illegal value format '%s' for hex" % v)


def check_int_dec(v):
    if not re.search(r'^[0-9xzXZ]+$', v):
        raise ValueError("Illegal value format '%s' for dec" % v)


def check_int_dec_pure(v):
    if not re.search(r'^[0-9]+$', v):
        raise ValueError("Illegal value format '%s' for dec" % v)


def check_int_oct(v):
    if not re.search(r'^[0-7xzXZ]+$', v):
        raise ValueError("Illegal value format '%s' for oct" % v)


def check_int_bin(v):
    if not re.search(r'^[01xzXZ]+$', v):
        raise ValueError("Illegal value format '%s' for bin" % v)


def str_to_value(s):
    targ = s.replace('_', '')

    match = re.search(r'h(.+)', targ)
    if match is not None:
        try:
            v = int(match.group(1), 16)
        except:
            v = match.group(1)
            check_int_hex(v)
        return v, 16

    match = re.search(r'd(.+)', targ)
    if match is not None:
        try:
            v = int(match.group(1), 10)
        except:
            v = match.group(1)
            check_int_dec(v)
        return v, 10

    match = re.search(r'o(.+)', targ)
    if match is not None:
        try:
            v = int(match.group(1), 8)
        except:
            v = match.group(1)
            check_int_oct(v)
        return v, 8

    match = re.search(r'b(.+)', targ)
    if match is not None:
        try:
            v = int(match.group(1), 2)
        except:
            v = match.group(1)
            check_int_bin(v)
        return v, 2

    try:
        v = int(targ, 10)
    except:
        v = targ
        check_int_dec_pure(v)
    return v, None


def str_to_width(s):
    targ = s.replace('_', '')
    match = re.search(r'(.+)\'h.+', targ)
    if match is not None:
        return int(match.group(1), 10)
    match = re.search(r'(.+)\'d.+', targ)
    if match is not None:
        return int(match.group(1), 10)
    match = re.search(r'(.+)\'o.+', targ)
    if match is not None:
        return int(match.group(1), 10)
    match = re.search(r'(.+)\'b.+', targ)
    if match is not None:
        return int(match.group(1), 10)
    return None


def get_signed(obj):
    if hasattr(obj, 'get_signed'):
        return obj.get_signed()
    return True


def max_width(left, right):
    if left is None and right is None:
        return None
    if left is None:
        return right
    if right is None:
        return left
    return Mux(left >= right, left, right)


def raw_value(v):
    if isinstance(v, Int):
        return v.value
    if isinstance(v, Float):
        return v.value
    if isinstance(v, Str):
        return v.value
    return v


class VeriloggenNode(object):
    """ Base class of Veriloggen AST object """

    def __init__(self):
        global global_object_counter
        self.object_id = global_object_counter
        global_object_counter += 1

    def __hash__(self):
        return hash((id(self), self.object_id))

    def __eq__(self, other):
        return (id(self), self.object_id) == (id(other), other.object_id)

    def __lt__(self, r):
        raise TypeError('Not allowed operation.')

    def __le__(self, r):
        raise TypeError('Not allowed operation.')

    # def __eq__(self, r):
    #    raise TypeError('Not allowed operation.')

    # def __ne__(self, r):
    #    raise TypeError('Not allowed operation.')

    def __ge__(self, r):
        raise TypeError('Not allowed operation.')

    def __gt__(self, r):
        raise TypeError('Not allowed operation.')

    def __add__(self, r):
        raise TypeError('Not allowed operation.')

    def __sub__(self, r):
        raise TypeError('Not allowed operation.')

    def __pow__(self, r):
        raise TypeError('Not allowed operation.')

    def __mul__(self, r):
        raise TypeError('Not allowed operation.')

    def __div__(self, r):
        raise TypeError('Not allowed operation.')

    def __truediv__(self, r):
        raise TypeError('Not allowed operation.')

    def __mod__(self, r):
        raise TypeError('Not allowed operation.')

    def __and__(self, r):
        raise TypeError('Not allowed operation.')

    def __or__(self, r):
        raise TypeError('Not allowed operation.')

    def __xor__(self, r):
        raise TypeError('Not allowed operation.')

    def __lshift__(self, r):
        raise TypeError('Not allowed operation.')

    def __rshift__(self, r):
        raise TypeError('Not allowed operation.')

    def __neg__(self):
        raise TypeError('Not allowed operation.')

    def __pos__(self):
        raise TypeError('Not allowed operation.')

    def __invert__(self):
        raise TypeError('Not allowed operation.')

    def __getitem__(self, r):
        raise TypeError('Not allowed operation.')


class _Numeric(VeriloggenNode):

    def __init__(self):
        VeriloggenNode.__init__(self)

    def __hash__(self):
        return hash((id(self), self.object_id))

    def __lt__(self, r):
        return LessThan(self, r)

    def __le__(self, r):
        return LessEq(self, r)

    def __eq__(self, r):
        return Eq(self, r)

    def __ne__(self, r):
        return NotEq(self, r)

    def __ge__(self, r):
        return GreaterEq(self, r)

    def __gt__(self, r):
        return GreaterThan(self, r)

    def __add__(self, r):
        return Plus(self, r)

    def __sub__(self, r):
        return Minus(self, r)

    def __pow__(self, r):
        return Power(self, r)

    def __mul__(self, r):
        return Times(self, r)

    def __div__(self, r):
        return Divide(self, r)

    def __truediv__(self, r):
        return Divide(self, r)

    def __mod__(self, r):
        return Mod(self, r)

    def __and__(self, r):
        return And(self, r)

    def __or__(self, r):
        return Or(self, r)

    def __xor__(self, r):
        return Xor(self, r)

    def __lshift__(self, r):
        return Sll(self, r)

    def __rshift__(self, r):
        return Srl(self, r)

    def __neg__(self):
        return Uminus(self)

    def __pos__(self):
        return Uplus(self)

    def __invert__(self):
        return Unot(self)

    def __getitem__(self, r):
        if isinstance(r, slice):
            size = self._len()

            right = r.start
            left = r.stop
            step = r.step

            if isinstance(left, Int):
                left = left.value
            if isinstance(right, Int):
                right = right.value
            if isinstance(step, Int):
                step = step.value

            if right is None:
                right = 0
            elif isinstance(right, int) and right < 0:
                right = size - abs(right)

            if left is None:
                left = size
            elif isinstance(left, int) and left < 0:
                left = size - abs(left)
            left -= 1

            if isinstance(left, int) and left < 0:
                raise ValueError("Illegal slice index: left = %d" % left)

            if step is None:
                return Slice(self, left, right)
            else:
                if not (isinstance(left, int) and
                        isinstance(right, int) and
                        isinstance(step, int)):
                    raise ValueError(
                        "Slice with step is not supported in Verilog Slice.")

                if step == 0:
                    raise ValueError("Illegal slice step: step = %d" % step)

                values = [Pointer(self, i)
                          for i in range(right, left + 1, step)]
                values.reverse()
                return Cat(*values)

        if isinstance(r, int) and r < 0:
            r = self.bit_length() - abs(r)

        return Pointer(self, r)

    def sra(self, r):  # shift right arithmetically
        return Sra(self, r)

    def repeat(self, times):
        return Repeat(self, times)

    def slice(self, msb, lsb):
        return Slice(self, msb, lsb)

    def bit_length(self):
        raise TypeError("bit_length() is not supported.")

    def get_signed(self):
        return False

    def __iter__(self):
        self.iter_size = len(self)
        self.iter_count = 0
        return self

    def __next__(self):
        if self.iter_count >= self.iter_size:
            raise StopIteration()

        ret = Pointer(self, self.iter_count)
        self.iter_count += 1
        return ret

    # for Python2
    def next(self):
        return self.__next__()

    def _len(self):
        if hasattr(self, 'length') and self.length is not None:
            ret = self.length
        else:
            ret = self.bit_length()
        return ret

    def __len__(self):
        ret = self._len()
        if not isinstance(ret, int):
            raise TypeError("Non int length.")
        return ret


class _Variable(_Numeric):

    def __init__(self, width=1, length=None, signed=False, value=None, initval=None, name=None,
                 module=None):
        _Numeric.__init__(self)
        self.name = name
        self.width = width
        self.width_msb = None
        self.width_lsb = None
        self.length = length
        self.length_msb = None
        self.length_lsb = None
        self.signed = signed
        self.value = value
        self.initval = initval
        self.module = module
        self.subst = []

    def write(self, value, blk=False, ldelay=None, rdelay=None):
        return Subst(self, value, blk=blk, ldelay=ldelay, rdelay=rdelay)

    def read(self):
        return self

    def assign(self, value):
        if self.module is None:
            raise ValueError(
                "Variable '%s' has no parent module information" % self.name)
        return self.module.Assign(self.write(value))

    def comb(self, value):
        return self.assign(value)

    def reset(self):
        return None

    def bit_length(self):
        return self.width

    def get_signed(self):
        return self.signed

    def _add_subst(self, s):
        self.subst.append(s)

    def _get_subst(self):
        return self.subst

    def _set_raw_width(self, msb, lsb):
        self.width_msb = msb
        self.width_lsb = lsb

    def _set_raw_length(self, msb, lsb):
        self.length_msb = msb
        self.length_lsb = lsb

    def _get_module(self):
        return self.module

    def __setattr__(self, attr, value):
        # when width or length is overwritten, msb and lsb values are reset.
        if attr == 'width':
            object.__setattr__(self, 'width_msb', None)
            object.__setattr__(self, 'width_lsb', None)
        if attr == 'length':
            object.__setattr__(self, 'length_msb', None)
            object.__setattr__(self, 'length_lsb', None)
        object.__setattr__(self, attr, value)

    def __str__(self):
        return self.name

    def __call__(self, value, blk=False, ldelay=None, rdelay=None):
        return self.write(value, blk=blk, ldelay=ldelay, rdelay=rdelay)


class Input(_Variable):
    pass


class Output(_Variable):
    pass


class Inout(_Variable):
    pass


class Tri(_Variable):
    pass


class Reg(_Variable):

    def assign(self, value):
        raise TypeError("Reg object accept no combinational assignment.")

    def reset(self):
        if self.initval is None:
            return None
        return self.write(self.initval)

    def add(self, r):
        return Subst(self, self + r)

    def sub(self, r):
        return Subst(self, self - r)

    def inc(self):
        return self.add(1)

    def dec(self):
        return self.sub(1)


class Wire(_Variable):

    def _add_subst(self, s):
        if len(self.subst) > 0:
            raise ValueError('Wire %s is already assigned.' % self.name)
        self.subst.append(s)


class Integer(_Variable):

    def reset(self):
        if self.initval is None:
            return None
        return self.write(self.initval)

    def add(self, r):
        return Subst(self, self + r)

    def sub(self, r):
        return Subst(self, self - r)

    def inc(self):
        return self.add(1)

    def dec(self):
        return self.sub(1)


class Real(_Variable):

    def reset(self):
        if self.initval is None:
            return None
        return self.write(self.initval)

    def add(self, r):
        return Subst(self, self + r)

    def sub(self, r):
        return Subst(self, self - r)

    def inc(self):
        return self.add(1)

    def dec(self):
        return self.sub(1)


class Genvar(_Variable):

    def add(self, r):
        return Subst(self, self + r)

    def sub(self, r):
        return Subst(self, self - r)

    def inc(self):
        return self.add(1)

    def dec(self):
        return self.sub(1)

# for undetermined identifier


class AnyType(_Variable):
    pass


class _ParameterVariable(_Variable):

    def __init__(self, value, width=None, signed=False, name=None, module=None):
        if isinstance(value, _ParameterVariable):
            value = value.value
        _Variable.__init__(self, width=width, signed=signed, value=value, name=name,
                           module=module)

    def bit_length(self):
        if self.width is None:
            return 32
        return self.width


class Parameter(_ParameterVariable):
    pass


class Localparam(_ParameterVariable):
    pass


class Supply(_ParameterVariable):
    pass


class _Constant(_Numeric):

    def __init__(self, value, width=None, base=None):
        _Numeric.__init__(self)
        self.value = value
        self.width = width
        self.base = base
        self._type_check_value(value)
        self._type_check_width(width)
        self._type_check_base(base)

    def _type_check_value(self, value): pass

    def _type_check_width(self, width): pass

    def _type_check_base(self, base): pass

    def __str__(self):
        return str(self.value)

    def bit_length(self):
        if self.width is None:
            return 32
        return self.width


class Int(_Constant):

    def __init__(self, value, width=None, base=None, signed=False):
        _Constant.__init__(self, value, width, base)
        if isinstance(value, int):
            self.value = value
            self.width = width
            self.base = base
            self.signed = signed
        else:
            self.value, self.base = str_to_value(value)
            if base is not None:
                self.base = base
            if not isinstance(self.value, int):
                if self.base is None:
                    check_int_dec_pure(self.value)
                elif self.base == 16:
                    check_int_hex(self.value)
                elif self.base == 10:
                    check_int_dec(self.value)
                elif self.base == 8:
                    check_int_oct(self.value)
                elif self.base == 2:
                    check_int_bin(self.value)
                else:
                    raise ValueError(
                        "Illegal base number %d for Int." % self.base)
            self.width = str_to_width(value) if width is None else width
            self.signed = str_to_signed(value) if signed == False else signed

    def _type_check_value(self, value):
        if not isinstance(value, (int, str)):
            raise TypeError(
                'value of Int must be int or str, not %s.' % str(type(value)))

    def _type_check_width(self, width):
        if width is None:
            return
        if not isinstance(width, int):
            raise TypeError('width of Int must be int, not %s.' %
                            str(type(width)))

    def _type_check_base(self, base):
        if base is None:
            return
        if not isinstance(base, int):
            raise TypeError('base of Int must be int, not %s.' %
                            str(type(base)))

    def __str__(self):
        value_list = []
        if self.width:
            value_list.append(str(self.width))

        if self.base is None:
            if self.signed:
                value_list.append("'sd")
            elif self.width:
                value_list.append("'d")
            if isinstance(self.value, str):
                value_list.append(self.value)
            else:
                value_list.append(str(self.value))
        elif self.base == 2:
            if self.signed:
                value_list.append("'sb")
            else:
                value_list.append("'b")
            if isinstance(self.value, str):
                value_list.append(self.value)
            else:
                value_list.append(bin(self .value).replace('0b', ''))
        elif self.base == 8:
            if self.signed:
                value_list.append("'so")
            else:
                value_list.append("'o")
            if isinstance(self.value, str):
                value_list.append(self.value)
            else:
                value_list.append(oct(self.value).replace('0o', ''))
        elif self.base == 10:
            if self.signed:
                value_list.append("'sd")
            else:
                value_list.append("'d")
            if isinstance(self.value, str):
                value_list.append(self.value)
            else:
                value_list.append(str(self.value))
        elif self.base == 16:
            if self.signed:
                value_list.append("'sh")
            else:
                value_list.append("'h")
            if isinstance(self.value, str):
                value_list.append(self.value)
            else:
                value_list.append(hex(self.value).replace('0x', ''))
        else:
            raise ValueError("Int.base must be 2, 8, 10, or 16")

        return ''.join(value_list)

    def get_signed(self):
        return self.signed


def IntX(width=None, base=None, signed=False):
    return Int("'hx", width, base, signed)


def IntZ(width=None, base=None, signed=False):
    return Int("'hz", width, base, signed)


class Float(_Constant):

    def __init__(self, value):
        _Constant.__init__(self, value, None, None)
        self.value = value
        self.width = 32

    def _type_check_value(self, value):
        if not isinstance(value, (float, int)):
            raise TypeError(
                'value of Float must be float, not %s.' % str(type(value)))

    def __str__(self):
        return str(self.value)

    def get_signed(self):
        return True


class Str(_Constant):

    def __init__(self, value):
        _Constant.__init__(self, value, None, None)
        self.value = value

    def _type_check_value(self, value):
        if not isinstance(value, str):
            raise TypeError('value of Str must be str, not %s.' %
                            str(type(value)))

    def __str__(self):
        return str(self.value)


class _Operator(_Numeric):

    def __init__(self):
        _Numeric.__init__(self)
        self.signed = False

    def get_signed(self):
        return self.signed


class _BinaryOperator(_Operator):

    def __init__(self, left, right):
        _Operator.__init__(self)
        self._type_check(left, right)
        self.left = left
        self.right = right
        self.signed = get_signed(self.left) and get_signed(self.right)

    def _type_check(self, left, right):
        if not isinstance(left, (_Numeric, bool, int, float, str)):
            raise TypeError(
                'BinaryOperator does not support Type %s' % str(type(left)))
        if not isinstance(right, (_Numeric, bool, int, float, str)):
            raise TypeError(
                'BinaryOperator does not support Type %s' % str(type(right)))

    def _get_module(self):
        if hasattr(self.left, '_get_module'):
            return self.left._get_module()
        if hasattr(self.right, '_get_module'):
            return self.right._get_module()
        return None

    def __str__(self):
        return ''.join(['(', str(self.left), ' ', op2mark(self.__class__.__name__), ' ',
                        str(self.right), ')'])

    def bit_length(self):
        left = self.left.bit_length()
        right = self.right.bit_length()
        return max_width(left, right) + 1

    def get_signed(self):
        return self.signed


class _UnaryOperator(_Operator):

    def __init__(self, right):
        _Operator.__init__(self)
        self._type_check(right)
        self.right = right
        self.signed = get_signed(self.right)

    def _type_check(self, right):
        if not isinstance(right, (_Numeric, bool, int, float, str)):
            raise TypeError(
                'BinaryOperator does not support Type %s' % str(type(right)))

    def _get_module(self):
        if hasattr(self.right, '_get_module'):
            return self.right._get_module()
        return None

    def __str__(self):
        return ''.join(['(', op2mark(self.__class__.__name__), str(self.right), ')'])

    def bit_length(self):
        return self.right.bit_length()


# class names must be same the ones in pyverilog.vparser.ast
class Power(_BinaryOperator):

    @staticmethod
    def op(left, right, lwidth, rwidth):
        return left ** right


class Times(_BinaryOperator):

    @staticmethod
    def op(left, right, lwidth, rwidth):
        return left * right

    def bit_length(self):
        left = self.left.bit_length()
        right = self.right.bit_length()
        return left + right


class Divide(_BinaryOperator):

    @staticmethod
    def op(left, right, lwidth, rwidth):
        return left // right

    def bit_length(self):
        left = self.left.bit_length()
        right = self.right.bit_length()
        return max_width(left, right)


class Mod(_BinaryOperator):

    @staticmethod
    def op(left, right, lwidth, rwidth):
        return left % right

    def bit_length(self):
        left = self.left.bit_length()
        right = self.right.bit_length()
        return max_width(left, right)


class Plus(_BinaryOperator):

    @staticmethod
    def op(left, right, lwidth, rwidth):
        return left + right


class Minus(_BinaryOperator):

    @staticmethod
    def op(left, right, lwidth, rwidth):
        return left - right


class Sll(_BinaryOperator):

    def __init__(self, left, right):
        _BinaryOperator.__init__(self, left, right)
        self.signed = False

    @staticmethod
    def op(left, right, lwidth, rwidth):
        return left << right

    def bit_length(self):
        left = self.left.bit_length()
        right = self.right.bit_length()
        return Int(2) ** right + left


class Srl(_BinaryOperator):

    def __init__(self, left, right):
        _BinaryOperator.__init__(self, left, right)
        self.signed = False

    @staticmethod
    def op(left, right, lwidth, rwidth):
        return left >> right

    def bit_length(self):
        left = self.left.bit_length()
        return left


class Sra(_BinaryOperator):

    def __init__(self, left, right):
        _BinaryOperator.__init__(self, left, right)
        self.signed = get_signed(self.left)

    @staticmethod
    def op(left, right, lwidth, rwidth):
        sign = left >= 0
        left = abs(left)
        ret = left >> right
        if not sign:
            return -1 * ret
        return ret

    def bit_length(self):
        left = self.left.bit_length()
        return left


class LessThan(_BinaryOperator):

    def __init__(self, left, right):
        _BinaryOperator.__init__(self, left, right)
        self.signed = False

    @staticmethod
    def op(left, right, lwidth, rwidth):
        return left < right

    def bit_length(self):
        return 1


class GreaterThan(_BinaryOperator):

    def __init__(self, left, right):
        _BinaryOperator.__init__(self, left, right)
        self.signed = False

    @staticmethod
    def op(left, right, lwidth, rwidth):
        return left > right

    def bit_length(self):
        return 1


class LessEq(_BinaryOperator):

    def __init__(self, left, right):
        _BinaryOperator.__init__(self, left, right)
        self.signed = False

    @staticmethod
    def op(left, right, lwidth, rwidth):
        return left <= right

    def bit_length(self):
        return 1


class GreaterEq(_BinaryOperator):

    def __init__(self, left, right):
        _BinaryOperator.__init__(self, left, right)
        self.signed = False

    @staticmethod
    def op(left, right, lwidth, rwidth):
        return left >= right

    def bit_length(self):
        return 1


class Eq(_BinaryOperator):

    def __init__(self, left, right):
        _BinaryOperator.__init__(self, left, right)
        self.signed = False

    @staticmethod
    def op(left, right, lwidth, rwidth):
        return left == right

    def bit_length(self):
        return 1


class NotEq(_BinaryOperator):

    def __init__(self, left, right):
        _BinaryOperator.__init__(self, left, right)
        self.signed = False

    @staticmethod
    def op(left, right, lwidth, rwidth):
        return left != right

    def bit_length(self):
        return 1


class Eql(_BinaryOperator):  # ===

    def __init__(self, left, right):
        _BinaryOperator.__init__(self, left, right)
        self.signed = False

    @staticmethod
    def op(left, right, lwidth, rwidth):
        return left == right

    def bit_length(self):
        return 1


class NotEql(_BinaryOperator):  # !==

    def __init__(self, left, right):
        _BinaryOperator.__init__(self, left, right)
        self.signed = False

    @staticmethod
    def op(left, right, lwidth, rwidth):
        return left != right

    def bit_length(self):
        return 1


class And(_BinaryOperator):

    def __init__(self, left, right):
        _BinaryOperator.__init__(self, left, right)
        self.signed = False

    @staticmethod
    def op(left, right, lwidth, rwidth):
        return left & right

    def bit_length(self):
        left = self.left.bit_length()
        right = self.right.bit_length()
        return max_width(left, right)


class Xor(_BinaryOperator):

    def __init__(self, left, right):
        _BinaryOperator.__init__(self, left, right)
        self.signed = False

    @staticmethod
    def op(left, right, lwidth, rwidth):
        return left ^ right

    def bit_length(self):
        left = self.left.bit_length()
        right = self.right.bit_length()
        return max_width(left, right)


class Xnor(_BinaryOperator):

    def __init__(self, left, right):
        _BinaryOperator.__init__(self, left, right)
        self.signed = False

    @staticmethod
    def op(left, right, lwidth, rwidth):
        width = max(lwidth, rwidth)
        value = ~(left ^ right)
        mask = 0
        for i in range(width):
            mask = (0x1 << i) | mask
        return mask & value

    def bit_length(self):
        left = self.left.bit_length()
        right = self.right.bit_length()
        return max_width(left, right)


class Or(_BinaryOperator):

    def __init__(self, left, right):
        _BinaryOperator.__init__(self, left, right)
        self.signed = False

    @staticmethod
    def op(left, right, lwidth, rwidth):
        return left | right

    def bit_length(self):
        left = self.left.bit_length()
        right = self.right.bit_length()
        return max_width(left, right)


class Land(_BinaryOperator):

    def __init__(self, left, right):
        _BinaryOperator.__init__(self, left, right)
        self.signed = False

    @staticmethod
    def op(left, right, lwidth, rwidth):
        if left and right:
            return True
        return False

    def bit_length(self):
        left = self.left.bit_length()
        right = self.right.bit_length()
        return max_width(left, right)


class Lor(_BinaryOperator):

    def __init__(self, left, right):
        _BinaryOperator.__init__(self, left, right)
        self.signed = False

    @staticmethod
    def op(left, right, lwidth, rwidth):
        if left or right:
            return True
        return False

    def bit_length(self):
        left = self.left.bit_length()
        right = self.right.bit_length()
        return max_width(left, right)


class Uplus(_UnaryOperator):

    @staticmethod
    def op(right, rwidth):
        return right


class Uminus(_UnaryOperator):

    @staticmethod
    def op(right, rwidth):
        return -right


class Ulnot(_UnaryOperator):

    def __init__(self, right):
        _UnaryOperator.__init__(self, right)
        self.signed = False

    @staticmethod
    def op(right, rwidth):
        return not right

    def bit_length(self):
        return 1


class Unot(_UnaryOperator):

    def __init__(self, right):
        _UnaryOperator.__init__(self, right)
        self.signed = False

    @staticmethod
    def op(right, rwidth):
        return ~right


class Uand(_UnaryOperator):

    def __init__(self, right):
        _UnaryOperator.__init__(self, right)
        self.signed = False

    @staticmethod
    def op(right, rwidth):
        for i in range(rwidth):
            v = (right >> i) & 0x1
            if not v:
                return False
        return True

    def bit_length(self):
        return 1


class Unand(_UnaryOperator):

    def __init__(self, right):
        _UnaryOperator.__init__(self, right)
        self.signed = False

    @staticmethod
    def op(right, rwidth):
        for i in range(rwidth):
            v = (right >> i) & 0x1
            if not v:
                return True
        return False

    def bit_length(self):
        return 1


class Uor(_UnaryOperator):

    def __init__(self, right):
        _UnaryOperator.__init__(self, right)
        self.signed = False

    @staticmethod
    def op(right, rwidth):
        for i in range(rwidth):
            v = (right >> i) & 0x1
            if v:
                return True
        return False

    def bit_length(self):
        return 1


class Unor(_UnaryOperator):

    def __init__(self, right):
        _UnaryOperator.__init__(self, right)
        self.signed = False

    @staticmethod
    def op(right, rwidth):
        for i in range(rwidth):
            v = (right >> i) & 0x1
            if v:
                return False
        return True

    def bit_length(self):
        return 1


class Uxor(_UnaryOperator):

    def __init__(self, right):
        _UnaryOperator.__init__(self, right)
        self.signed = False

    @staticmethod
    def op(right, rwidth):
        ret = False
        for i in range(rwidth):
            v = (right >> i) & 0x1
            ret = ret ^ v
        return ret

    def bit_length(self):
        return 1


class Uxnor(_UnaryOperator):

    def __init__(self, right):
        _UnaryOperator.__init__(self, right)
        self.signed = False

    @staticmethod
    def op(right, rwidth):
        ret = True
        for i in range(rwidth):
            v = (right >> i) & 0x1
            ret = ret ^ v
        return ret

    def bit_length(self):
        return 1


# alias
def Not(*args):
    return Ulnot(*args)


def AndList(*args):
    if len(args) == 0:
        raise ValueError("LandList requires at least one argument.")
    if len(args) == 1:
        return args[0]
    left = args[0]
    for right in args[1:]:
        left = Land(left, right)
    return left


def OrList(*args):
    if len(args) == 0:
        raise ValueError("LorList requires at least one argument.")
    if len(args) == 1:
        return args[0]
    left = args[0]
    for right in args[1:]:
        left = Lor(left, right)
    return left

Ands = AndList
Ors = OrList


class _SpecialOperator(_Operator):

    def __init__(self, *args, **kwargs):
        _Operator.__init__(self)
        self.args = args
        self.kwargs = kwargs
        self.signed = False

    def _get_module(self):
        return None


class Pointer(_SpecialOperator):

    def __init__(self, var, pos):
        _SpecialOperator.__init__(self)
        self.var = var
        self.pos = pos
        self.subst = []
        self._type_check_var(var)

    def write(self, value, blk=False, ldelay=None, rdelay=None):
        return Subst(self, value, blk=blk, ldelay=ldelay, rdelay=rdelay)

    def read(self):
        return self

    def bit_length(self):
        if isinstance(self.var, _Variable) and self.var.length is not None:
            return self.var.bit_length()
        return 1

    def assign(self, value):
        module = self._get_module()
        if module is None:
            raise ValueError("This Pointer has no parent module information")
        return module.Assign(self.write(value))

    def _type_check_var(self, var):
        if not isinstance(var, (_Variable, Scope, Pointer)):
            raise TypeError(
                'var of Pointer must be Variable, not %s' % str(type(var)))

    def _add_subst(self, s):
        self.subst.append(s)

    def _get_subst(self):
        return self.subst

    def _get_module(self):
        if not hasattr(self.var, '_get_module'):
            return None
        return self.var._get_module()

    def __str__(self):
        return ''.join([str(self.var), '[', str(self.pos), ']'])

    def __call__(self, value, blk=False, ldelay=None, rdelay=None):
        return self.write(value, blk=blk, ldelay=ldelay, rdelay=rdelay)

    @staticmethod
    def op(var, pos):
        return (var >> pos) & 0x1


class Slice(_SpecialOperator):

    def __init__(self, var, msb, lsb):
        _SpecialOperator.__init__(self)
        self.var = var
        self.msb = msb
        self.lsb = lsb
        self.subst = []
        self._type_check_var(var)

    def write(self, value, blk=False, ldelay=None, rdelay=None):
        return Subst(self, value, blk=blk, ldelay=ldelay, rdelay=rdelay)

    def read(self):
        return self

    def bit_length(self):
        return self.msb - self.lsb + 1

    def assign(self, value):
        module = self._get_module()
        if module is None:
            raise ValueError("This Slice has no parent module information")
        return module.Assign(self.write(value))

    def _type_check_var(self, var):
        if not isinstance(var, (_Variable, Scope)):
            raise TypeError(
                'var of Slice must be Variable, not %s' % str(type(var)))

    def _add_subst(self, s):
        self.subst.append(s)

    def _get_subst(self):
        return self.subst

    def _get_module(self):
        if not hasattr(self.var, '_get_module'):
            return None
        return self.var._get_module()

    def __str__(self):
        return ''.join([str(self.var), '[', str(self.msb), ':', str(self.lsb), ']'])

    def __call__(self, value, blk=False, ldelay=None, rdelay=None):
        return self.write(value, blk=blk, ldelay=ldelay, rdelay=rdelay)

    @staticmethod
    def op(var, msb, lsb):
        mask = 0
        for i in range(msb - lsb + 1):
            mask = (mask << 1) | 0x1
        return (var >> lsb) & mask


class Cat(_SpecialOperator):

    def __init__(self, *vars):
        _SpecialOperator.__init__(self)
        self.vars = tuple(vars)
        self.subst = []

    def write(self, value, blk=False, ldelay=None, rdelay=None):
        return Subst(self, value, blk=blk, ldelay=ldelay, rdelay=rdelay)

    def read(self):
        return self

    def bit_length(self):
        values = [v.bit_length() for v in self.vars]
        ret = values[0]
        for v in values[1:]:
            ret = ret + v
        return ret

    def assign(self, value):
        module = self._get_module()
        if module is None:
            raise ValueError("This Cat has no parent module information")
        return module.Assign(self.write(value))

    def _add_subst(self, s):
        self.subst.append(s)

    def _get_subst(self):
        return self.subst

    def _get_module(self):
        for var in self.vars:
            if hasattr(var, '_get_module'):
                return var._get_module()
        return None

    def __str__(self):
        ret = []
        ret.append('{')
        for v in self.vars:
            ret.append(str(v))
            ret.append(', ')
        ret.pop()
        ret.append('}')
        return ''.join(ret)

    def __call__(self, value, blk=False, ldelay=None, rdelay=None):
        return self.write(value, blk=blk, ldelay=ldelay, rdelay=rdelay)

    @staticmethod
    def op(vars, widths):
        ret = 0
        for var, width in zip(vars, widths):
            ret = (ret << width) | var
        return ret


class Repeat(_SpecialOperator):

    def __init__(self, var, times):
        _SpecialOperator.__init__(self)
        self.var = var
        self.times = times

    def bit_length(self):
        return self.var.bit_length() * self.times

    def __str__(self):
        return ''.join(['{', str(self.times), '{', str(self.var), '}}'])

    @staticmethod
    def op(var, width, times):
        ret = 0
        for i in times:
            ret = (ret << width) | var
        return ret


class Cond(_SpecialOperator):

    def __init__(self, condition, true_value, false_value):
        _SpecialOperator.__init__(self)
        self.condition = condition
        self.true_value = true_value
        self.false_value = false_value
        self.signed = get_signed(
            self.true_value) or get_signed(self.false_value)

    def bit_length(self):
        t = self.true_value.bit_length()
        f = self.false_value.bit_length()
        return max_width(t, f)

    def __str__(self):
        return ''.join(['(', str(self.condition), ')?',
                        str(self.true_value), ' : ', str(self.false_value)])

    @staticmethod
    def op(condition, true_value, false_value):
        if condition:
            return true_value
        else:
            return false_value


def Mux(condition, true_value, false_value):
    # return the result immediately if the condition can be resolved now
    if isinstance(condition, (bool, int, float, str, list, tuple)):
        return true_value if condition else false_value
    return Cond(condition, true_value, false_value)


class Sensitive(VeriloggenNode):

    def __init__(self, name):
        VeriloggenNode.__init__(self)
        self.name = name


class Posedge(Sensitive):
    pass


class Negedge(Sensitive):
    pass


class SensitiveAll(Sensitive):

    def __init__(self):
        Sensitive.__init__(self, 'all')


class Subst(VeriloggenNode):

    def __init__(self, left, right, blk=False, ldelay=None, rdelay=None):
        VeriloggenNode.__init__(self)
        self._type_check_left(left)
        self._type_check_right(right)
        self.left = left
        self.right = right
        self.blk = blk
        self.ldelay = ldelay
        self.rdelay = rdelay
        self.left._add_subst(self)

    def _type_check_left(self, left):
        if not isinstance(left, VeriloggenNode):
            raise TypeError(
                "left must be VeriloggenNode, not '%s'" % str(type(left)))

    def _type_check_right(self, right):
        if not isinstance(right, (VeriloggenNode, int, float, bool, str)):
            raise TypeError(
                "right must be VeriloggenNode, not '%s'" % str(type(right)))

    def overwrite_right(self, right):
        self.right = right

    def __str__(self):
        return ''.join([str(self.left), ' <- ', str(self.right)])


class Always(VeriloggenNode):

    def __init__(self, *sensitivity):
        VeriloggenNode.__init__(self)
        self.sensitivity = tuple(sensitivity)
        self.statement = None

    def set_statement(self, *statement):
        if self.statement is not None:
            raise ValueError("Statement is already assigned.")
        self.statement = tuple(statement)
        return self

    def __call__(self, *statement):
        return self.set_statement(*statement)


class Assign(VeriloggenNode):

    def __init__(self, statement):
        VeriloggenNode.__init__(self)
        self.statement = statement


class Initial(VeriloggenNode):

    def __init__(self, *statement):
        VeriloggenNode.__init__(self)
        self.statement = tuple(statement)

    def add(self, *statement):
        if self.statement is None:
            return self.set_statement(*statement)
        self.statement = tuple(self.statement + statement)
        return self


class If(VeriloggenNode):

    def __init__(self, condition):
        VeriloggenNode.__init__(self)
        self.condition = condition
        self.true_statement = None
        self.false_statement = None

        self.root = self
        self.next_call = None

    def set_true_statement(self, *statement):
        self.true_statement = tuple(statement)
        return self.root

    def set_false_statement(self, *statement):
        self.false_statement = tuple(statement)
        return self.root

    def Else(self, *statement):
        if self.next_call is not None:
            return self.next_call.Else(*statement)
        if self.false_statement is None:
            return self.set_false_statement(*statement)
        raise ValueError("False statement is already assigned.")

    def Elif(self, condition):
        next_If = If(condition)
        next_If.root = self.root
        self.Else(next_If)
        self.next_call = next_If
        return self

    def __call__(self, *args):
        if self.next_call is not None:
            return self.next_call(*args)
        if self.true_statement is None:
            return self.set_true_statement(*args)
        if self.false_statement is None:
            return self.set_false_statement(*args)
        raise ValueError(
            "True statement and False statement are already assigned.")


class For(VeriloggenNode):

    def __init__(self, pre, condition, post):
        VeriloggenNode.__init__(self)
        self.pre = pre
        self.condition = condition
        self.post = post
        self.statement = None

    def set_statement(self, *statement):
        self.statement = tuple(statement)
        return self

    def add(self, *statement):
        if self.statement is None:
            return self.set_statement(*statement)
        self.statement = tuple(self.statement + statement)
        return self

    def __call__(self, *args):
        if self.statement is None:
            return self.set_statement(*args)
        raise ValueError("Statement body is already assigned.")


class While(VeriloggenNode):

    def __init__(self, condition):
        VeriloggenNode.__init__(self)
        self.condition = condition
        self.statement = None

    def set_statement(self, *statement):
        self.statement = tuple(statement)
        return self

    def add(self, *statement):
        if self.statement is None:
            return self.set_statement(*statement)
        self.statement = tuple(self.statement + statement)
        return self

    def __call__(self, *args):
        if self.statement is None:
            return self.set_statement(*args)
        raise ValueError("Statement body is already assigned.")


class Case(VeriloggenNode):

    def __init__(self, comp):
        VeriloggenNode.__init__(self)
        self.comp = comp
        self.statement = None
        self.last = False

    def _type_check_statement(self, *statement):
        for s in statement:
            if not isinstance(s, When):
                raise TypeError(
                    "Case statement requires When() object as statement list.")
            if self.last:
                raise ValueError("When() with None condition must be last.")
            if s.condition is None:
                self.last = True

    def set_statement(self, *statement):
        self._type_check_statement(*statement)
        self.statement = tuple(statement)
        return self

    def add(self, *statement):
        if self.statement is None:
            return self.set_statement(*statement)
        self._type_check_statement(*statement)
        self.statement = tuple(self.statement + statement)
        return self

    def __call__(self, *args):
        if self.statement is None:
            return self.set_statement(*args)
        raise ValueError("Case statement list is already assigned.")


class Casex(Case):
    pass


class When(VeriloggenNode):

    def __init__(self, *condition):
        VeriloggenNode.__init__(self)
        self._type_check_condition(*condition)
        self.condition = None if len(condition) == 0 or condition[
            0] is None else tuple(condition)
        self.statement = None

    def _type_check_condition(self, *args):
        if len(args) == 0:
            return
        if len(args) == 1 and args[0] is None:
            return
        for i, a in enumerate(args):
            if a is None:
                raise ValueError(
                    "None condition must not mixed in When() statement.")
            if isinstance(a, (_Numeric, bool, int, float, str)):
                continue
            raise TypeError(
                "Condition must be _Numeric object, not '%s'" % str(type(a)))

    def _type_check_statement(self, *args):
        pass

    def set_statement(self, *statement):
        self._type_check_statement(*statement)
        self.statement = tuple(statement)
        return self

    def add(self, *statement):
        if self.statement is None:
            return self.set_statement(*statement)
        self._type_check_statement(*statement)
        self.statement = tuple(self.statement + statement)
        return self

    def __call__(self, *args):
        if self.statement is None:
            return self.set_statement(*args)
        raise ValueError("Statement body is already assigned.")


def PatternIf(*patterns):
    root = None
    prev = None
    length = len(patterns)

    if len(patterns) == 1 and isinstance(patterns, (tuple, list)):
        patterns = patterns[0]

    for i, (cond, stmt) in enumerate(patterns):
        if not isinstance(stmt, (tuple, list)):
            stmt = tuple([stmt])

        body = If(cond)(stmt) if cond is not None else stmt

        if root is None:
            root = body
        else:
            prev.Else(body)

        prev = body

        if cond is None:
            if i < length - 1:
                raise ValueError("Too many patterns after None condition.")
            break

    return root


def PatternMux(*patterns):
    prev = None

    if len(patterns) == 1 and isinstance(patterns, (tuple, list)):
        patterns = patterns[0]

    for i, (cond, stmt) in enumerate(reversed(patterns)):
        if prev is None and cond is not None:
            raise ValueError('Last pattern requires a None condition.')
        if prev is not None and cond is None:
            raise ValueError('Non-last pattern requires a condition.')
        prev = Mux(cond, stmt, prev) if cond is not None else stmt

    return prev


class ScopeIndex(VeriloggenNode):

    def __init__(self, name, index):
        VeriloggenNode.__init__(self)
        self.name = name
        self.index = index


class Scope(_Numeric):

    def __init__(self, *args):
        _Numeric.__init__(self)
        self.args = tuple(args)
        if not args:
            raise ValueError("Scope requires at least one argument.")

    def bit_length(self):
        return self.args[-1].bit_length()


class SystemTask(_Numeric):

    def __init__(self, cmd, *args):
        _Numeric.__init__(self)
        self.cmd = cmd
        self.args = tuple(args)

    def bit_length(self):
        if self.cmd == 'signed':
            return self.args[0].bit_length()
        raise TypeError("bit_length() is not supported.")


def Systask(cmd, *args):
    return SingleStatement(SystemTask(cmd, *args))

# frequently-used system task


def Display(*args):
    return Systask('display', *args)


def Write(*args):
    return Systask('write', *args)


def Finish():
    return Systask('finish')


class Event(VeriloggenNode):

    def __init__(self, *sensitivity):
        VeriloggenNode.__init__(self)
        self.sensitivity = sensitivity


class Wait(VeriloggenNode):

    def __init__(self, condition):
        VeriloggenNode.__init__(self)
        self.condition = condition
        self.statement = None

    def set_statement(self, *statement):
        if self.statement is not None:
            raise ValueError("Statement is already assigned.")
        self.statement = tuple(statement)
        return self

    def __call__(self, *statement):
        return self.set_statement(*statement)


class Forever(VeriloggenNode):

    def __init__(self, *statement):
        VeriloggenNode.__init__(self)
        self.statement = tuple(statement)


class Delay(VeriloggenNode):

    def __init__(self, value):
        VeriloggenNode.__init__(self)
        self.value = value


class SingleStatement(VeriloggenNode):

    def __init__(self, statement):
        VeriloggenNode.__init__(self)
        self.statement = statement

numerical_types = (_Numeric, int, bool, float, str)



from __future__ import absolute_import
from __future__ import print_function
import veriloggen
import types_fixed_mul

expected_verilog = """
module blinkled #
(
  parameter WIDTH = 8
)
(
  input CLK,
  input RST,
  output reg [WIDTH-1:0] LED
);

  reg [32-1:0] count;

  always @(posedge CLK) begin
    if(RST) begin
      count <= 0;
    end else begin
      if(count == 1023) begin
        count <= 0;
      end else begin
        count <= count + 1;
      end
    end
  end


  always @(posedge CLK) begin
    if(RST) begin
      LED <= 0;
    end else begin
      if(count == 1023) begin
        LED <= LED + 1;
      end 
    end
  end

  reg [32-1:0] a;
  reg [32-1:0] b;
  reg [32-1:0] c;
  reg [32-1:0] d;
  reg signed [32-1:0] sa;
  reg signed [32-1:0] sb;
  reg signed [32-1:0] sc;
  reg signed [32-1:0] sd;

  always @(posedge CLK) begin
    if(RST) begin
      a <= 16;
      b <= 512;
      c <= 131072;
      d <= 128;
      sa <= 16;
      sb <= 512;
      sc <= 131072;
      sd <= 128;
    end else begin
      a <= ($signed(a) * $signed(b) >> 4) >> 4;
      b <= $signed(a) * $signed(b) >> 4;
      c <= ($signed(a) * $signed(b) >> 4) << 8;
      d <= ($signed(b) * $signed(a) >> 4) >> 2;
      sa <= ($signed(sa) * $signed(sb) >>> 4) >>> 4;
      sb <= $signed(sa) * $signed(sb) >>> 4;
      sc <= ($signed(sa) * $signed(sb) >>> 4) << 8;
      sd <= ($signed(sb) * $signed(sa) >>> 4) >>> 2;
    end
  end


endmodule
"""

def test():
    veriloggen.reset()
    test_module = types_fixed_mul.mkLed()
    code = test_module.to_verilog()

    from pyverilog.vparser.parser import VerilogParser
    from pyverilog.ast_code_generator.codegen import ASTCodeGenerator
    parser = VerilogParser()
    expected_ast = parser.parse(expected_verilog)
    codegen = ASTCodeGenerator()
    expected_code = codegen.visit(expected_ast)

    assert(expected_code == code)



from __future__ import absolute_import
from __future__ import print_function
import sys
import os

# the next line can be removed after installation
sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.dirname(
    os.path.dirname(os.path.dirname(os.path.abspath(__file__)))))))

from veriloggen import *
import veriloggen.thread as vthread
import veriloggen.types.axi as axi


def mkLed():
    m = Module('blinkled')
    clk = m.Input('CLK')
    rst = m.Input('RST')

    datawidth = 32
    addrwidth = 10
    myaxi = vthread.AXIM(m, 'myaxi', clk, rst, datawidth)
    myram = vthread.RAM(m, 'myram', clk, rst, datawidth, addrwidth)

    def blink():
        size = 256 * 2
        offset = 1024 * 4

        # write
        for i in range(size):
            wdata = i
            myram.write(i, wdata)

        laddr = 0
        gaddr = offset
        myram.dma_write(myaxi, laddr, gaddr, size)
        print('dma_write: [%d] -> [%d]' % (laddr, gaddr))

        # overwrite
        for i in range(size):
            wdata = 128
            myram.write(i, wdata)

        laddr = 0
        gaddr = offset + size * 4
        myram.dma_write(myaxi, laddr, gaddr, size)
        print('dma_write: [%d] -> [%d]' % (laddr, gaddr))

        # read
        all_ok = True

        laddr = 0
        gaddr = offset
        myram.dma_read(myaxi, laddr, gaddr, size)
        print('dma_read:  [%d] <- [%d]' % (laddr, gaddr))

        for i in range(size):
            rdata = myram.read(i)
            if rdata != i:
                print('rdata[%d] = %d' % (i, rdata))
                all_ok = False

        # read
        laddr = 0
        gaddr = offset + size * 4
        myram.dma_read(myaxi, laddr, gaddr, size)
        print('dma_read:  [%d] <- [%d]' % (laddr, gaddr))

        for i in range(size):
            rdata = myram.read(i)
            if rdata != 128:
                print('rdata[%d] = %d' % (i, rdata))
                all_ok = False

        if all_ok:
            print('ALL OK')

    th = vthread.Thread(m, 'th_blink', clk, rst, blink)
    fsm = th.start()

    return m


def mkTest():
    m = Module('test')

    # target instance
    led = mkLed()

    # copy paras and ports
    params = m.copy_params(led)
    ports = m.copy_sim_ports(led)

    clk = ports['CLK']
    rst = ports['RST']

    memory = axi.AxiMemoryModel(m, 'memory', clk, rst)
    memory.connect(ports, 'myaxi')

    uut = m.Instance(led, 'uut',
                     params=m.connect_params(led),
                     ports=m.connect_ports(led))

    simulation.setup_waveform(m, uut)
    simulation.setup_clock(m, clk, hperiod=5)
    init = simulation.setup_reset(m, rst, m.make_reset(), period=100)

    init.add(
        Delay(200000),
        Systask('finish'),
    )

    return m

if __name__ == '__main__':
    test = mkTest()
    verilog = test.to_verilog('tmp.v')
    print(verilog)

    sim = simulation.Simulator(test)
    rslt = sim.run()
    print(rslt)



from __future__ import absolute_import
from __future__ import print_function
import sys
import os
import math
import cmath

# the next line can be removed after installation
sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__)))))

from veriloggen import *
import veriloggen.dataflow as dataflow
import veriloggen.types.fixed as fixed

def _log2(v): 
    return math.log(v, 2)

log2 = math.log2 if hasattr(math, 'log2') else _log2

#-------------------------------------------------------------------------------
def complex_add(x, y):
    a = x[0]
    b = x[1]
    c = y[0]
    d = y[1]
    return (a + c), (b + d)

def complex_sub(x, y):
    a = x[0]
    b = x[1]
    c = y[0]
    d = y[1]
    return (a - c), (b - d)

def complex_mult(x, y):
    a = x[0]
    b = x[1]
    c = y[0]
    d = y[1]
    # (a + jb) * (c + jd) = ac - bd + j(ad + bc)
    ac = a * c
    bd = b * d
    ad = a * d
    bc = b * c
    re = ac - bd
    im = ad + bc
    return re, im

def radix2(x, y, c):
    d0 = complex_add(x, y)
    d1 = complex_sub(x, y)
    r0 = d0 # as-is
    r1 = complex_mult(d1, c)
    return r0, r1

#-------------------------------------------------------------------------------
def gen_weight(n):
    c = cmath.rect(1, -2 * cmath.pi / n)
    weight = []
    
    for i in range(int(log2(n))):
        offset = 0
        offset_count = 0
        for k in range(int(n / 2)):
            first = k + offset
            p = (first * (2 ** i)) % int(n / 2)
            _w = c ** p
            w = (_w.real, _w.imag)
            weight.append(w)
            
            offset_count += 1
            if offset_count == (n >> (i+1)):
                offset_count = 0
                offset += (n >> (i+1))
                
    return weight

def fft_weight(din, n, weight):
    for i in range(int(log2(n))):
        offset = 0
        offset_count = 0
        
        ndin0 = []
        ndin1 = []
        ndin = []

        for k in range(int(n / 2)):
            first = k + offset
            second = k + offset + (n >> (i+1))
            
            w = weight.pop(0)
            r0, r1 = radix2(din[first], din[second], w)
            
            ndin0.append(r0)
            ndin1.append(r1)
            
            offset_count += 1
            if offset_count == (n >> (i+1)):
                offset_count = 0
                offset += (n >> (i+1))
                ndin.extend(ndin0)
                ndin.extend(ndin1)
                ndin0 = []
                ndin1 = []
            
        din = ndin

    # reorder by bit-inversed index
    ret = []
    for i in range(len(din)):
        # bit-inversion
        fm = '{:0' + str(int(log2(n))) + 'b}'
        index = int(fm.format(i)[::-1], 2)
        #print(i, '->', index)
        re, im = din[index]
        ret.append( (re, im) )

    return ret

def fft(din, n):
    weight = gen_weight(n)
    return fft_weight(din, n, weight)
    
#-------------------------------------------------------------------------------
def mkFFT(n, datawidth=16, point=8):
    din = [ (dataflow.Variable('din' + str(i) + 're', width=datawidth, point=point, signed=True),
             dataflow.Variable('din' + str(i) + 'im', width=datawidth, point=point, signed=True))
            for i in range(n) ]

    weight = [ (dataflow.Variable('weight' + str(i) + 're', width=datawidth,
                                  point=point, signed=True),
                dataflow.Variable('weight' + str(i) + 'im', width=datawidth,
                                  point=point, signed=True))
               for i in range(int(n * log2(n) / 2)) ]
    
    # call software-defined method
    rslt = fft_weight(din, n, weight)

    vars = []
    for i, (re, im) in enumerate(rslt):
        re.output('dout' + str(i) + 're')
        im.output('dout' + str(i) + 'im')
        vars.append(re)
        vars.append(im)

    df = dataflow.Dataflow(*vars)
    m = df.to_module('fft')

    #try:
    #    df.draw_graph()
    #except:
    #    print('Dataflow graph could not be generated.', file=sys.stderr)

    return m

#-------------------------------------------------------------------------------
def mkTest(n=8, datawidth=16, point=8):
    m = Module('test')
    
    main = mkFFT(n, datawidth, point)
    
    params = m.copy_params(main)
    ports = m.copy_sim_ports(main)

    clk = ports['CLK']
    rst = ports['RST']

    din = [ (ports['din' + str(i) + 're'], ports['din' + str(i) + 'im']) for i in range(n) ]
    dout = [ (ports['dout' + str(i) + 're'], ports['dout' + str(i) + 'im']) for i in range(n) ]
    weight = [ (ports['weight' + str(i) + 're'], ports['weight' + str(i) + 'im'])
               for i in range(int(n * log2(n) / 2)) ]

    _din = [ (m.WireLike(re, name='_' + re.name, width=datawidth-point),
              m.WireLike(im, name='_' + im.name, width=datawidth-point))
             for re, im in din ]
    _dout = [ (m.WireLike(re, name='_' + re.name, width=datawidth-point),
               m.WireLike(im, name='_' + im.name, width=datawidth-point))
              for re, im in dout ]
    _weight = [ (m.WireLike(re, name='_' + re.name, width=datawidth-point),
                 m.WireLike(im, name='_' + im.name, width=datawidth-point))
                for re, im in weight ]
    
    for (lre, lim), (rre, rim) in zip(_din, din):
        m.Assign( lre(fixed.fixed_to_int(rre, point)) )
        m.Assign( lim(fixed.fixed_to_int(rim, point)) )
    
    for (lre, lim), (rre, rim) in zip(_dout, dout):
        m.Assign( lre(fixed.fixed_to_int(rre, point)) )
        m.Assign( lim(fixed.fixed_to_int(rim, point)) )
    
    for (lre, lim), (rre, rim) in zip(_weight, weight):
        m.Assign( lre(fixed.fixed_to_int(rre, point)) )
        m.Assign( lim(fixed.fixed_to_int(rim, point)) )
    
    uut = m.Instance(main, 'uut',
                     params=m.connect_params(main),
                     ports=m.connect_ports(main))

    reset_done = m.Reg('reset_done', initval=0)
    reset_stmt = []
    reset_stmt.append( reset_done(0) )
    for i, (re, im) in enumerate(din):
        reset_stmt.append( re(fixed.to_fixed(i, point)) )
        reset_stmt.append( im(fixed.to_fixed(i, point)) )

    weight_value = gen_weight(n)
        
    for i, (re, im) in enumerate(weight):
        wr, wi = weight_value[i]
        reset_stmt.append( re(fixed.to_fixed(wr, point)) )
        reset_stmt.append( im(fixed.to_fixed(wi, point)) )
    
    simulation.setup_waveform(m, uut, *(_din + _dout + _weight))
    simulation.setup_clock(m, clk, hperiod=5)
    init = simulation.setup_reset(m, rst, reset_stmt, period=100)

    nclk = simulation.next_clock
    
    init.add(
        Delay(1000),
        reset_done(1),
        nclk(clk),
        Delay(10000),
        Systask('finish'),
    )

    def dump(name, v, point):
        return Systask('display', name + '= %f', fixed.fixed_to_real(v, point))
        
    send_fsm = FSM(m, 'send_fsm', clk, rst)
    send_fsm.goto_next(cond=reset_done)

    #for i, (re, im) in enumerate(weight):
    #    send_fsm.add( dump('w[%d]re' % i, re, point) )
    #    send_fsm.add( dump('w[%d]im' % i, im, point) )
    
    for i, (re, im) in enumerate(din):
        send_fsm.add( re(fixed.to_fixed(i, point)) )
        send_fsm.add( im(fixed.to_fixed(i, point)) )
    #    send_fsm.add( dump('din[%d]re' % i, re, point), delay=1 )
    #    send_fsm.add( dump('din[%d]im' % i, im, point), delay=1 )
        
    send_fsm.goto_next()

    for i, (re, im) in enumerate(din):
        send_fsm.add( re(fixed.to_fixed(0, point)) )
        send_fsm.add( im(fixed.to_fixed(0, point)) )
        
    send_fsm.goto_next()

    for _ in range(100):
        #for i, (re, im) in enumerate(dout):
        #    send_fsm.add( dump('dout[%d]re' % i, re, point), delay=1 )
        #    send_fsm.add( dump('dout[%d]im' % i, im, point), delay=1 )
            
        send_fsm.goto_next()

    send_fsm.add( Systask('finish') )

    send_fsm.make_always()

    return m

if __name__ == '__main__':
    n = 8
    point = 8
    test = mkTest(n, point=point)
    verilog = test.to_verilog('tmp.v')
    #print(verilog)

    # run simulator (Icarus Verilog)
    sim = simulation.Simulator(test)
    rslt = sim.run()
    print(rslt)

    ## only target RTL
    #main = mkFFT(n)
    #verilog = main.to_verilog('tmp.v')
    #print(verilog)

    din = [ (i, i) for i in range(n) ]
    rslt = fft(din, n)
    for r in rslt:
        print( complex(round(r[0] * (2 ** point), 5), round(r[1] * (2 ** point), 5)), ':',
               complex(round(r[0], 5), round(r[1], 5)) )



from __future__ import absolute_import
from __future__ import print_function
import veriloggen
import bram

expected_verilog = """
module test
(

);

  reg CLK;
  reg RST;

  main
  uut
  (
    .CLK(CLK),
    .RST(RST)
  );


  initial begin
    $dumpfile("uut.vcd");
    $dumpvars(0, uut, CLK, RST);
  end


  initial begin
    CLK = 0;
    forever begin
      #5 CLK = !CLK;
    end
  end


  initial begin
    RST = 0;
    #100;
    RST = 1;
    #100;
    RST = 0;
    #100000;
    $finish;
  end


endmodule



module main
(
  input CLK,
  input RST
);

  reg [14-1:0] bram_addr0;
  reg [32-1:0] bram_din0;
  reg bram_we0;
  wire [32-1:0] bram_dout0;
  reg [14-1:0] bram_addr1;
  reg [32-1:0] bram_din1;
  reg bram_we1;
  wire [32-1:0] bram_dout1;

  BRAM2
  bram_inst
  (
    .CLK(CLK),
    .ADDR0(bram_addr0),
    .DIN0(bram_din0),
    .WE0(bram_we0),
    .DOUT0(bram_dout0),
    .ADDR1(bram_addr1),
    .DIN1(bram_din1),
    .WE1(bram_we1),
    .DOUT1(bram_dout1)
  );

  reg [32-1:0] fsm;
  localparam fsm_init = 0;
  reg [32-1:0] count;
  localparam fsm_1 = 1;
  localparam fsm_2 = 2;

  always @(posedge CLK) begin
    if(RST) begin
      fsm <= fsm_init;
      count <= 0;
    end else begin
      case(fsm)
        fsm_init: begin
          bram_addr0 <= 0;
          bram_din0 <= 0;
          bram_we0 <= 0;
          bram_addr1 <= 0;
          bram_din1 <= 0;
          bram_we1 <= 0;
          fsm <= fsm_1;
        end
        fsm_1: begin
          bram_addr0 <= count + 0;
          bram_din0 <= count;
          bram_we0 <= 1;
          count <= count + 1;
          bram_addr1 <= count + 64;
          bram_din1 <= count;
          bram_we1 <= 1;
          count <= count + 1;
          if(count == 63) begin
            fsm <= fsm_2;
          end 
        end
      endcase
    end
  end


endmodule



module BRAM2
(
  input CLK,
  input [14-1:0] ADDR0,
  input [32-1:0] DIN0,
  input WE0,
  output [32-1:0] DOUT0,
  input [14-1:0] ADDR1,
  input [32-1:0] DIN1,
  input WE1,
  output [32-1:0] DOUT1
);

  reg [14-1:0] delay_ADDR0;
  reg [14-1:0] delay_ADDR1;
  reg [32-1:0] mem [0:16384-1];

  always @(posedge CLK) begin
    if(WE0) begin
      mem[ADDR0] <= DIN0;
    end 
    delay_ADDR0 <= ADDR0;
  end

  assign DOUT0 = mem[delay_ADDR0];

  always @(posedge CLK) begin
    if(WE1) begin
      mem[ADDR1] <= DIN1;
    end 
    delay_ADDR1 <= ADDR1;
  end

  assign DOUT1 = mem[delay_ADDR1];

endmodule
"""

def test():
    veriloggen.reset()
    test_module = bram.mkTest()
    code = test_module.to_verilog()

    from pyverilog.vparser.parser import VerilogParser
    from pyverilog.ast_code_generator.codegen import ASTCodeGenerator
    parser = VerilogParser()
    expected_ast = parser.parse(expected_verilog)
    codegen = ASTCodeGenerator()
    expected_code = codegen.visit(expected_ast)

    assert(expected_code == code)



from __future__ import absolute_import
from __future__ import print_function
import sys
import os
import math
from functools import reduce

# the next line can be removed after installation
sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__)))))

from veriloggen import *
import veriloggen.dataflow as dataflow
import veriloggen.types.ram as ram
import veriloggen.types.fixed as fixed

def stencil(coe, data):
    data = map(lambda x,y: x*y, data, coe)
    rslt = reduce(lambda x,y: x+y, data)
    return rslt
    
def mkStencilPipeline2D(coe=None, size=3, width=32, point=16):
    # size-port stream inputs
    iports = [ dataflow.Variable('idata%d'%i, valid='ivalid%d'%i,
                                 width=width, point=point, signed=True)
               for i in range(size) ]

    if coe is None:
        coe = [ [ dataflow.Constant(1.0/(1.0*size*size), point=point) for i in range(size) ]
                for j in range(size) ]
        
    # source data
    data = [ [ d.prev(j) for j in range(size) ] for d in iports ]
    
    # from 2D list to 1D list
    data_list = []
    coe_list = []
    for d, c in zip(data, coe):
        data_list.extend(d)
        coe_list.extend(c)

    # computation by calling standard method
    rslt = stencil(coe_list, data_list)
    
    rslt.output('odata', valid='ovalid')
    
    df = dataflow.Dataflow(rslt)
    m = df.to_module('stencil_pipeline_2d')

    #try:
    #    df.draw_graph()
    #except:
    #    print('Dataflow graph could not be generated.', file=sys.stderr)

    return m

def mkStencil(n=16, size=3, datawidth=32, point=16, coe_test=False):
    m = Module('stencil')

    addrwidth = int(math.log(n, 2))

    clk = m.Input('CLK')
    rst = m.Input('RST')

    start = m.Input('start')
    busy = m.OutputReg('busy', initval=0)
    
    done = m.TmpReg(initval=0)

    # external RAM I/F
    ext_src_rams = [ ram.RAMSlaveInterface(m, 'ext_src_ram%d' % i,
                                              datawidth=datawidth, addrwidth=addrwidth)
                      for i in range(size) ]
    ext_dst_ram = ram.RAMSlaveInterface(m, 'ext_dst_ram',
                                           datawidth=datawidth, addrwidth=addrwidth)
    
    # RAM
    addrwidth = int(math.log(n, 2)) * 2
    
    src_rams = [ ram.SyncRAMManager(m, 'src_ram%d' % i, clk, rst,
                                    datawidth=datawidth, addrwidth=addrwidth, numports=2)
                  for i in range(size) ]
                            
    dst_ram = ram.SyncRAMManager(m, 'dst_ram', clk, rst,
                                 datawidth=datawidth, addrwidth=addrwidth, numports=2)

    # connect RAM I/Fs
    for src_ram, ext_src_ram in zip(src_rams, ext_src_rams):
        src_ram[1].connect(ext_src_ram)

    dst_ram[1].connect(ext_dst_ram)

    # read FSM
    read_fsm = FSM(m, 'read_fsm', clk, rst)
    read_count = m.Reg('read_count', 32, initval=0)
    read_addr = m.Reg('read_addr', 32, initval=0)


    read_fsm(
        read_addr(0),
        read_count(0),
        busy(0)
    )

    read_fsm.If(start)(
        busy(1)
    )

    read_fsm.Then().goto_next()

    read_fsm(
        read_addr.inc(),
        read_count.inc()
    )
        
    idata = []
    ivalid = []
    for i, src_ram in enumerate(src_rams):
        src_ram.disable_write(0)
        rdata, rvalid = src_ram.read(0, read_addr, read_fsm)
        idata.append(rdata)
        ivalid.append(rvalid)

    read_fsm.If(read_count == n - 1)(
        read_addr(0),
        read_count(0)
    )

    read_fsm.Then().goto_next()

    read_fsm.If(done)(
        busy(0)
    )

    read_fsm.Then().goto_init()

    read_fsm.make_always()


    # instance
    odata = m.Wire('odata', datawidth)
    ovalid = m.Wire('ovalid')
    
    ports = []
    ports.append( ('CLK', clk) )
    ports.append( ('RST', rst) )    
    
    for i, (d, v) in enumerate(zip(idata, ivalid)):
        ports.append( ('idata%d' % i, d) )
        ports.append( ('ivalid%d' % i, v) )

    ports.append( ('odata', odata) )
    ports.append( ('ovalid', ovalid) )

    coe = None
    if coe_test:
        coe = [ [ dataflow.Constant(1, point=point) for i in range(size) ]
                for j in range(size) ]
        point = 0
        
    st = mkStencilPipeline2D(size=3, width=datawidth, point=point, coe=coe)
    m.Instance(st, 'inst_stencil', ports=ports)

    skip_offset = int(math.floor(size/2))
    
    # write FSM
    write_fsm = FSM(m, 'write_fsm', clk, rst)
    write_count = m.Reg('write_count', 32, initval=0)
    write_addr = m.Reg('write_addr', 32, initval=skip_offset)

    write_fsm(
        done(0)
    )
    
    write_fsm.If(Ands(ovalid, write_count > skip_offset))(
        write_addr.inc()
    )
    
    dst_ram.write(0, write_addr, odata, write_fsm.then)
    
    write_fsm.If(ovalid)(
        write_count.inc(),
    )
    
    write_fsm.If(write_count == n)(
        write_count(0),
        write_addr(skip_offset),
        done(1)
    )
    write_fsm.Then().goto_init()

    write_fsm.make_always()

    return m

def mkTest(n=16, size=3, datawidth=32, point=16, coe_test=False):
    if coe_test:
        point = 0
    
    m = Module('test')

    addrwidth = int(math.log(n, 2))

    main = mkStencil(n, size, datawidth, point, coe_test)

    params = m.copy_params(main)
    ports = m.copy_sim_ports(main)
    
    clk = ports['CLK']
    rst = ports['RST']

    start = ports['start']
    busy = ports['busy']
    
    uut = m.Instance(main, 'uut',
                     params=m.connect_params(main),
                     ports=m.connect_ports(main))

    reset_done = m.Reg('reset_done', initval=0)
    reset_stmt = []
    reset_stmt.append( reset_done(0) )
    reset_stmt.append( start(0) )

    
    # src RAM
    for i in range(3):
        addr = ports['ext_src_ram%d_addr' % i]
        rdata = ports['ext_src_ram%d_rdata' % i]
        wdata = ports['ext_src_ram%d_wdata' % i]
        wenable = ports['ext_src_ram%d_wenable' % i]
        reset_stmt.append(addr(0))
        reset_stmt.append(wdata(0))
        reset_stmt.append(wenable(0))

    # dst RAM
    addr = ports['ext_dst_ram_addr']
    rdata = ports['ext_dst_ram_rdata']
    wdata = ports['ext_dst_ram_wdata']
    wenable = ports['ext_dst_ram_wenable']
    reset_stmt.append(addr(2))
    reset_stmt.append(wdata(0))
    reset_stmt.append(wenable(0))
    

    simulation.setup_waveform(m, uut)
    simulation.setup_clock(m, clk, hperiod=5)
    init = simulation.setup_reset(m, rst, reset_stmt, period=100)

    nclk = simulation.next_clock

    init.add(
        Delay(1000),
        reset_done(1),
        nclk(clk),
        Delay(100000),
        Systask('finish'),
    )

    fsm = FSM(m, 'fsm', clk, rst)
    
    fsm.goto_next(cond=reset_done)

    for i in range(3):
        addr = ports['ext_src_ram%d_addr' % i]
        fsm.add( addr(-1) )

    fsm.goto_next()

    for i in range(3):
        addr = ports['ext_src_ram%d_addr' % i]
        rdata = ports['ext_src_ram%d_rdata' % i]
        wdata = ports['ext_src_ram%d_wdata' % i]
        wenable = ports['ext_src_ram%d_wenable' % i]
        next_addr = (addr+1) % (n*n)
        fsm.add( addr.inc() )
        fsm.add( wdata(fixed.FixedConst(90, point).raw) )
        fsm.add( wenable(1) )
        fsm.add( wenable(0), cond=AndList(wenable, addr==2**addrwidth-1) )

    fsm.goto_next(cond=AndList(wenable, ports['ext_src_ram0_addr']==2**addrwidth-1))
    
    fsm.goto_next(cond=Not(busy))
    
    fsm.add( start(1) )
    fsm.add( start(0), delay=1 )
    fsm.goto_next()
    
    fsm.goto_next(cond=busy)

    fsm.goto_next(cond=Not(busy))

    fsm.add( Systask('finish') )

    fsm.make_always()

    return m

if __name__ == '__main__':
    n = 16
    #test = mkTest(n, coe_test=True)
    test = mkTest(n)
    verilog = test.to_verilog('tmp.v')
    #print(verilog)

    # run simulator (Icarus Verilog)
    sim = simulation.Simulator(test)
    rslt = sim.run()
    print(rslt)



from __future__ import absolute_import
from __future__ import print_function
import veriloggen
import fsm_tmpfsm

expected_verilog = """
module test #
(
  parameter INTERVAL = 16
)
(

);

  reg CLK;
  reg RST;
  wire [8-1:0] LED;

  blinkled
  #(
    .INTERVAL(INTERVAL)
  )
  uut
  (
    .CLK(CLK),
    .RST(RST),
    .LED(LED)
  );


  initial begin
    CLK = 0;
    forever begin
      #5 CLK = !CLK;
    end
  end


  initial begin
    RST = 0;
    #100;
    RST = 1;
    #100;
    RST = 0;
    #1000;
    $finish;
  end


endmodule



module blinkled #
(
  parameter INTERVAL = 16
)
(
  input CLK,
  input RST,
  output reg [8-1:0] LED
);

  reg [32-1:0] _tmp_0;
  reg [32-1:0] _tmp_fsm_0;
  localparam _tmp_fsm_0_init = 0;
  localparam _tmp_fsm_0_1 = 1;

  always @(posedge CLK) begin
    if(RST) begin
      _tmp_fsm_0 <= _tmp_fsm_0_init;
      _tmp_0 <= 0;
      LED <= 0;
    end else begin
      case(_tmp_fsm_0)
        _tmp_fsm_0_init: begin
          if(_tmp_0 < INTERVAL - 1) begin
            _tmp_0 <= _tmp_0 + 1;
          end else begin
            _tmp_0 <= 0;
          end
          if(!(_tmp_0 < INTERVAL - 1)) begin
            _tmp_fsm_0 <= _tmp_fsm_0_1;
          end 
        end
        _tmp_fsm_0_1: begin
          LED <= LED + 1;
          _tmp_fsm_0 <= _tmp_fsm_0_init;
        end
      endcase
    end
  end


endmodule
"""

def test():
    veriloggen.reset()
    test_module = fsm_tmpfsm.mkTest()
    code = test_module.to_verilog()

    from pyverilog.vparser.parser import VerilogParser
    from pyverilog.ast_code_generator.codegen import ASTCodeGenerator
    parser = VerilogParser()
    expected_ast = parser.parse(expected_verilog)
    codegen = ASTCodeGenerator()
    expected_code = codegen.visit(expected_ast)

    assert(expected_code == code)



from __future__ import absolute_import
from __future__ import print_function
import veriloggen
import systemtask

expected_verilog = """
module blinkled #
  ( 
   parameter WIDTH = 8
  )
  ( 
   input CLK, 
   input RST,
   output reg [WIDTH-1:0] LED
  );
  reg [32-1:0] count;
  always @(posedge CLK) begin
    if(RST) begin
      count <= 0;
    end else begin
      if(count == 1023) begin
        count <= 0;
      end else begin
        count <= count + 1;
      end
    end
  end
  always @(posedge CLK) begin
    if(RST) begin
      LED <= 0;
    end else begin
      if(count == 1023) begin
        LED <= LED + 1;
        $display("led:%x", LED);
      end 
    end
  end
endmodule
"""

def test_led():
    test_module = systemtask.mkLed()
    code = test_module.to_verilog()

    from pyverilog.vparser.parser import VerilogParser
    from pyverilog.ast_code_generator.codegen import ASTCodeGenerator
    parser = VerilogParser()
    expected_ast = parser.parse(expected_verilog)
    codegen = ASTCodeGenerator()
    expected_code = codegen.visit(expected_ast)

    assert(expected_code == code)



from __future__ import absolute_import
from __future__ import print_function
import sys
import os

# the next line can be removed after installation
sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__)))))))

from veriloggen import *

def mkLed():
    m = Module('blinkled')
    width = m.Parameter('WIDTH', 8)
    clk = m.Input('CLK')
    rst = m.Input('RST')
    led = m.OutputReg('LED', width, initval=0)
    count = m.Reg('count', 32, initval=0)

    """
    led = 0
    while True:
        count = 0
        while count < 1024:
            count += 1
        led += 1
    """
    
    fsm = FSM(m, 'fsm', clk, rst)

    fsm( led(0) )
    fsm.goto_next()
    
    init = fsm.current
    fsm( count(0) )
    fsm.goto_next()
    
    head = fsm.current
    cond = count < 1024
    fsm.goto_next(cond=cond)
    
    fsm( count.inc() )
    fsm.goto_next()
    
    fsm( count.inc() )
    fsm.goto(head)
    fsm.inc()
    
    tail = fsm.current
    fsm.goto_from(head, tail, cond=Not(cond))
    
    fsm( led.inc() )
    fsm.goto(init)
    
    #fsm.make_always()
    # make_alway() is called when to_veirlog() is called.
    m.add_hook(fsm.make_always, args=(), kwargs={})
    
    return m

def mkTop():
    m = Module('top')
    led = mkLed()
    params = m.copy_params(led)
    ports = m.copy_ports(led)
    m.Instance(led, 'inst_' + led.name,
               connect_same_name(*params.values()),
               connect_same_name(*ports.values()))
    return m

if __name__ == '__main__':
    top = mkTop()
    # to_verilog() method is immuatable.
    # Hooked methods are called for the copied object to keep the internal state.
    dummy = top.to_verilog()
    verilog = top.to_verilog()
    print(verilog)



from __future__ import absolute_import
from __future__ import print_function
import sys
import os
import math

# the next line can be removed after installation
sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.dirname(
    os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))))))

from veriloggen import *
import veriloggen.types.fifo as fifo


def mkMain(n=128, datawidth=32, numports=2):
    m = Module('main')

    clk = m.Input('CLK')
    rst = m.Input('RST')

    addrwidth = int(math.log(n, 2))

    myfifo = fifo.Fifo(m, 'myfifo', clk, rst, datawidth, addrwidth)

    # example how to access BRAM
    count = m.Reg('count', 32, initval=0)
    sum = m.Reg('sum', 32, initval=0)

    fsm = FSM(m, 'fsm', clk, rst)

    fsm(
        count(0),
    )

    fsm.goto_next()

    step = 16

    ack, ready = myfifo.enq(count, cond=fsm)

    fsm.If(ready)(
        count.inc()
    )

    fsm.If(ack)(
        Systask('display', 'count=%d space=%d has_space=%d',
                myfifo.count, myfifo.space, myfifo.has_space())
    )

    fsm.If(Ands(ready, count == step - 1)).goto_next()

    fsm(
        count(0)
    )

    fsm.goto_next()

    data, valid = myfifo.deq(cond=fsm)

    fsm.If(valid)(
        sum(sum + data),
        count.inc(),
        Systask('write', 'count=%d space=%d has_space=%d ',
                myfifo.count, myfifo.space, myfifo.has_space())
    )

    fsm.Then().Delay(1)(
        Systask('display', "sum=%d", sum)
    )

    fsm.If(count == step).goto_next()

    fsm.make_always()

    return m


def mkTest():
    m = Module('test')

    # target instance
    main = mkMain()

    # copy paras and ports
    params = m.copy_params(main)
    ports = m.copy_sim_ports(main)

    clk = ports['CLK']
    rst = ports['RST']

    uut = m.Instance(main, 'uut',
                     params=m.connect_params(main),
                     ports=m.connect_ports(main))

    simulation.setup_waveform(m, uut, m.get_vars())
    simulation.setup_clock(m, clk, hperiod=5)
    init = simulation.setup_reset(m, rst, m.make_reset(), period=100)

    init.add(
        Delay(1000 * 100),
        Systask('finish'),
    )

    return m

if __name__ == '__main__':
    test = mkTest()
    verilog = test.to_verilog('tmp.v')
    print(verilog)

    sim = simulation.Simulator(test)
    rslt = sim.run()
    print(rslt)

    # sim.view_waveform()



from __future__ import absolute_import
from __future__ import print_function
import veriloggen
import dataflow_regionadd_filter

expected_verilog = """
module test;

  reg CLK;
  reg RST;
  reg [32-1:0] xdata;
  reg xvalid;
  wire xready;
  wire signed [32-1:0] zdata;
  wire zvalid;
  reg zready;
  wire [1-1:0] vdata;
  wire vvalid;
  reg vready;

  main
  uut
  (
    .CLK(CLK),
    .RST(RST),
    .xdata(xdata),
    .xvalid(xvalid),
    .xready(xready),
    .zdata(zdata),
    .zvalid(zvalid),
    .zready(zready),
    .vdata(vdata),
    .vvalid(vvalid),
    .vready(vready)
  );

  reg reset_done;

  initial begin
    $dumpfile("uut.vcd");
    $dumpvars(0, uut);
  end


  initial begin
    CLK = 0;
    forever begin
      #5 CLK = !CLK;
    end
  end


  initial begin
    RST = 0;
    reset_done = 0;
    xdata = 0;
    xvalid = 0;
    zready = 0;
    #100;
    RST = 1;
    #100;
    RST = 0;
    #1000;
    reset_done = 1;
    @(posedge CLK);
    #1;
    #10000;
    $finish;
  end

  reg [32-1:0] xfsm;
  localparam xfsm_init = 0;
  reg [32-1:0] _tmp_0;
  localparam xfsm_1 = 1;
  localparam xfsm_2 = 2;
  localparam xfsm_3 = 3;
  localparam xfsm_4 = 4;
  localparam xfsm_5 = 5;
  localparam xfsm_6 = 6;
  localparam xfsm_7 = 7;
  localparam xfsm_8 = 8;
  localparam xfsm_9 = 9;
  localparam xfsm_10 = 10;
  localparam xfsm_11 = 11;
  localparam xfsm_12 = 12;
  localparam xfsm_13 = 13;
  localparam xfsm_14 = 14;
  localparam xfsm_15 = 15;
  localparam xfsm_16 = 16;
  localparam xfsm_17 = 17;
  localparam xfsm_18 = 18;
  localparam xfsm_19 = 19;
  localparam xfsm_20 = 20;
  localparam xfsm_21 = 21;
  localparam xfsm_22 = 22;
  localparam xfsm_23 = 23;
  localparam xfsm_24 = 24;

  always @(posedge CLK) begin
    if(RST) begin
      xfsm <= xfsm_init;
      _tmp_0 <= 0;
    end else begin
      case(xfsm)
        xfsm_init: begin
          xvalid <= 0;
          if(reset_done) begin
            xfsm <= xfsm_1;
          end 
        end
        xfsm_1: begin
          xfsm <= xfsm_2;
        end
        xfsm_2: begin
          xfsm <= xfsm_3;
        end
        xfsm_3: begin
          xfsm <= xfsm_4;
        end
        xfsm_4: begin
          xfsm <= xfsm_5;
        end
        xfsm_5: begin
          xfsm <= xfsm_6;
        end
        xfsm_6: begin
          xfsm <= xfsm_7;
        end
        xfsm_7: begin
          xfsm <= xfsm_8;
        end
        xfsm_8: begin
          xfsm <= xfsm_9;
        end
        xfsm_9: begin
          xfsm <= xfsm_10;
        end
        xfsm_10: begin
          xfsm <= xfsm_11;
        end
        xfsm_11: begin
          xvalid <= 1;
          xfsm <= xfsm_12;
        end
        xfsm_12: begin
          if(xready) begin
            xdata <= xdata + 1;
          end 
          if(xready) begin
            _tmp_0 <= _tmp_0 + 1;
          end 
          if((_tmp_0 == 5) && xready) begin
            xvalid <= 0;
          end 
          if((_tmp_0 == 5) && xready) begin
            xfsm <= xfsm_13;
          end 
        end
        xfsm_13: begin
          xfsm <= xfsm_14;
        end
        xfsm_14: begin
          xfsm <= xfsm_15;
        end
        xfsm_15: begin
          xfsm <= xfsm_16;
        end
        xfsm_16: begin
          xfsm <= xfsm_17;
        end
        xfsm_17: begin
          xfsm <= xfsm_18;
        end
        xfsm_18: begin
          xfsm <= xfsm_19;
        end
        xfsm_19: begin
          xfsm <= xfsm_20;
        end
        xfsm_20: begin
          xfsm <= xfsm_21;
        end
        xfsm_21: begin
          xfsm <= xfsm_22;
        end
        xfsm_22: begin
          xfsm <= xfsm_23;
        end
        xfsm_23: begin
          xvalid <= 1;
          if(xready) begin
            xdata <= xdata + 1;
          end 
          if(xready) begin
            _tmp_0 <= _tmp_0 + 1;
          end 
          if((_tmp_0 == 100) && xready) begin
            xvalid <= 0;
          end 
          if((_tmp_0 == 100) && xready) begin
            xfsm <= xfsm_24;
          end 
        end
      endcase
    end
  end

  reg [32-1:0] zfsm;
  localparam zfsm_init = 0;
  localparam zfsm_1 = 1;
  localparam zfsm_2 = 2;
  localparam zfsm_3 = 3;
  localparam zfsm_4 = 4;
  localparam zfsm_5 = 5;
  localparam zfsm_6 = 6;
  localparam zfsm_7 = 7;
  localparam zfsm_8 = 8;

  always @(posedge CLK) begin
    if(RST) begin
      zfsm <= zfsm_init;
    end else begin
      case(zfsm)
        zfsm_init: begin
          zready <= 0;
          if(reset_done) begin
            zfsm <= zfsm_1;
          end 
        end
        zfsm_1: begin
          zfsm <= zfsm_2;
        end
        zfsm_2: begin
          if(zvalid) begin
            zready <= 1;
          end 
          if(zvalid) begin
            zfsm <= zfsm_3;
          end 
        end
        zfsm_3: begin
          zready <= 0;
          zfsm <= zfsm_4;
        end
        zfsm_4: begin
          zready <= 0;
          zfsm <= zfsm_5;
        end
        zfsm_5: begin
          zready <= 0;
          zfsm <= zfsm_6;
        end
        zfsm_6: begin
          zready <= 0;
          zfsm <= zfsm_7;
        end
        zfsm_7: begin
          zready <= 0;
          zfsm <= zfsm_8;
        end
        zfsm_8: begin
          zfsm <= zfsm_2;
        end
      endcase
    end
  end

  reg [32-1:0] vfsm;
  localparam vfsm_init = 0;
  localparam vfsm_1 = 1;
  localparam vfsm_2 = 2;
  localparam vfsm_3 = 3;
  localparam vfsm_4 = 4;
  localparam vfsm_5 = 5;
  localparam vfsm_6 = 6;
  localparam vfsm_7 = 7;
  localparam vfsm_8 = 8;

  always @(posedge CLK) begin
    if(RST) begin
      vfsm <= vfsm_init;
    end else begin
      case(vfsm)
        vfsm_init: begin
          vready <= 0;
          if(reset_done) begin
            vfsm <= vfsm_1;
          end 
        end
        vfsm_1: begin
          vfsm <= vfsm_2;
        end
        vfsm_2: begin
          if(vvalid) begin
            vready <= 1;
          end 
          if(vvalid) begin
            vfsm <= vfsm_3;
          end 
        end
        vfsm_3: begin
          vready <= 0;
          vfsm <= vfsm_4;
        end
        vfsm_4: begin
          vready <= 0;
          vfsm <= vfsm_5;
        end
        vfsm_5: begin
          vready <= 0;
          vfsm <= vfsm_6;
        end
        vfsm_6: begin
          vready <= 0;
          vfsm <= vfsm_7;
        end
        vfsm_7: begin
          vready <= 0;
          vfsm <= vfsm_8;
        end
        vfsm_8: begin
          vfsm <= vfsm_2;
        end
      endcase
    end
  end


  always @(posedge CLK) begin
    if(reset_done) begin
      if(xvalid && xready) begin
        $display("xdata=%d", xdata);
      end 
      if(zvalid && zready) begin
        $display("zdata=%d", zdata);
      end 
      if(vvalid && vready) begin
        $display("vdata=%d", vdata);
      end 
    end 
  end


endmodule



module main
(
  input CLK,
  input RST,
  input [32-1:0] xdata,
  input xvalid,
  output xready,
  output signed [32-1:0] zdata,
  output zvalid,
  input zready,
  output [1-1:0] vdata,
  output vvalid,
  input vready
);

  reg [32-1:0] _tmp_data_0;
  reg _tmp_valid_0;
  wire _tmp_ready_0;
  assign _tmp_ready_0 = (_tmp_ready_1 || !_tmp_valid_1) && _tmp_valid_0;
  reg [1-1:0] _tmp_data_1;
  reg _tmp_valid_1;
  wire _tmp_ready_1;
  assign _tmp_ready_1 = (_tmp_ready_5 || !_tmp_valid_5) && (_tmp_valid_2 && _tmp_valid_1) && ((_tmp_ready_6 || !_tmp_valid_6) && _tmp_valid_1);
  reg [32-1:0] _tmp_data_2;
  reg _tmp_valid_2;
  wire _tmp_ready_2;
  assign xready = (_tmp_ready_2 || !_tmp_valid_2) && xvalid;
  assign _tmp_ready_2 = (_tmp_ready_5 || !_tmp_valid_5) && (_tmp_valid_2 && _tmp_valid_1);
  reg [1-1:0] _tmp_data_3;
  reg [1-1:0] _tmp_data_4;
  reg [32-1:0] _tmp_data_5;
  reg _tmp_valid_5;
  wire _tmp_ready_5;
  assign _tmp_ready_5 = (_tmp_ready_7 || !_tmp_valid_7) && (_tmp_valid_6 && _tmp_valid_5);
  reg [1-1:0] _tmp_data_6;
  reg _tmp_valid_6;
  wire _tmp_ready_6;
  assign _tmp_ready_6 = (_tmp_ready_7 || !_tmp_valid_7) && (_tmp_valid_6 && _tmp_valid_5) && ((_tmp_ready_8 || !_tmp_valid_8) && _tmp_valid_6);
  reg signed [32-1:0] _tmp_data_7;
  reg _tmp_valid_7;
  wire _tmp_ready_7;
  reg [1-1:0] _tmp_data_8;
  reg _tmp_valid_8;
  wire _tmp_ready_8;
  assign zdata = _tmp_data_7;
  assign zvalid = _tmp_valid_7;
  assign _tmp_ready_7 = zready;
  assign vdata = _tmp_data_8;
  assign vvalid = _tmp_valid_8;
  assign _tmp_ready_8 = vready;

  always @(posedge CLK) begin
    if(RST) begin
      _tmp_data_0 <= 1'd0;
      _tmp_valid_0 <= 0;
      _tmp_data_1 <= 0;
      _tmp_valid_1 <= 0;
      _tmp_data_2 <= 0;
      _tmp_valid_2 <= 0;
      _tmp_data_3 <= 0;
      _tmp_data_4 <= 0;
      _tmp_data_5 <= 1'd0;
      _tmp_valid_5 <= 0;
      _tmp_data_6 <= 0;
      _tmp_valid_6 <= 0;
      _tmp_data_7 <= 0;
      _tmp_valid_7 <= 0;
      _tmp_data_8 <= 0;
      _tmp_valid_8 <= 0;
    end else begin
      if((_tmp_ready_0 || !_tmp_valid_0) && 1 && 1) begin
        _tmp_data_0 <= (_tmp_data_0 >= 3)? 0 : _tmp_data_0 + 2'd1;
      end 
      if(_tmp_valid_0 && _tmp_ready_0) begin
        _tmp_valid_0 <= 0;
      end 
      if((_tmp_ready_0 || !_tmp_valid_0) && 1) begin
        _tmp_valid_0 <= 1;
      end 
      if((_tmp_ready_1 || !_tmp_valid_1) && _tmp_ready_0 && _tmp_valid_0) begin
        _tmp_data_1 <= _tmp_data_0 == 3'd3;
      end 
      if(_tmp_valid_1 && _tmp_ready_1) begin
        _tmp_valid_1 <= 0;
      end 
      if((_tmp_ready_1 || !_tmp_valid_1) && _tmp_ready_0) begin
        _tmp_valid_1 <= _tmp_valid_0;
      end 
      if((_tmp_ready_2 || !_tmp_valid_2) && xready && xvalid) begin
        _tmp_data_2 <= xdata;
      end 
      if(_tmp_valid_2 && _tmp_ready_2) begin
        _tmp_valid_2 <= 0;
      end 
      if((_tmp_ready_2 || !_tmp_valid_2) && xready) begin
        _tmp_valid_2 <= xvalid;
      end 
      if(_tmp_valid_1 && _tmp_ready_1) begin
        _tmp_data_3 <= _tmp_data_1;
      end 
      if(_tmp_valid_1 && _tmp_ready_1) begin
        _tmp_data_4 <= _tmp_data_3;
      end 
      if((_tmp_ready_5 || !_tmp_valid_5) && (_tmp_ready_2 && _tmp_ready_1) && (_tmp_valid_2 && _tmp_valid_1)) begin
        _tmp_data_5 <= _tmp_data_5 + _tmp_data_2;
      end 
      if(_tmp_valid_5 && _tmp_ready_5) begin
        _tmp_valid_5 <= 0;
      end 
      if((_tmp_ready_5 || !_tmp_valid_5) && (_tmp_ready_2 && _tmp_ready_1)) begin
        _tmp_valid_5 <= _tmp_valid_2 && _tmp_valid_1;
      end 
      if((_tmp_ready_5 || !_tmp_valid_5) && (_tmp_ready_2 && _tmp_ready_1) && (_tmp_valid_2 && _tmp_valid_1) && _tmp_data_4) begin
        _tmp_data_5 <= 1'd0 + _tmp_data_2;
      end 
      if((_tmp_ready_6 || !_tmp_valid_6) && _tmp_ready_1 && _tmp_valid_1) begin
        _tmp_data_6 <= _tmp_data_3;
      end 
      if(_tmp_valid_6 && _tmp_ready_6) begin
        _tmp_valid_6 <= 0;
      end 
      if((_tmp_ready_6 || !_tmp_valid_6) && _tmp_ready_1) begin
        _tmp_valid_6 <= _tmp_valid_1;
      end 
      if((_tmp_ready_7 || !_tmp_valid_7) && (_tmp_ready_6 && _tmp_ready_5) && (_tmp_valid_6 && _tmp_valid_5)) begin
        _tmp_data_7 <= (_tmp_data_6)? _tmp_data_5 : 1'd0;
      end 
      if(_tmp_valid_7 && _tmp_ready_7) begin
        _tmp_valid_7 <= 0;
      end 
      if((_tmp_ready_7 || !_tmp_valid_7) && (_tmp_ready_6 && _tmp_ready_5)) begin
        _tmp_valid_7 <= _tmp_valid_6 && _tmp_valid_5;
      end 
      if((_tmp_ready_8 || !_tmp_valid_8) && _tmp_ready_6 && _tmp_valid_6) begin
        _tmp_data_8 <= _tmp_data_6;
      end 
      if(_tmp_valid_8 && _tmp_ready_8) begin
        _tmp_valid_8 <= 0;
      end 
      if((_tmp_ready_8 || !_tmp_valid_8) && _tmp_ready_6) begin
        _tmp_valid_8 <= _tmp_valid_6;
      end 
    end
  end


endmodule
"""

def test():
    veriloggen.reset()
    test_module = dataflow_regionadd_filter.mkTest()
    code = test_module.to_verilog()

    from pyverilog.vparser.parser import VerilogParser
    from pyverilog.ast_code_generator.codegen import ASTCodeGenerator
    parser = VerilogParser()
    expected_ast = parser.parse(expected_verilog)
    codegen = ASTCodeGenerator()
    expected_code = codegen.visit(expected_ast)

    assert(expected_code == code)



from __future__ import absolute_import
from __future__ import print_function
import veriloggen
import types_ram_manager_seq

expected_verilog = """
module test;

  reg CLK;
  reg RST;

  main
  uut
  (
    .CLK(CLK),
    .RST(RST)
  );


  initial begin
    $dumpfile("uut.vcd");
    $dumpvars(0, uut, CLK, RST);
  end


  initial begin
    CLK = 0;
    forever begin
      #5 CLK = !CLK;
    end
  end


  initial begin
    RST = 0;
    #100;
    RST = 1;
    #100;
    RST = 0;
    #100000;
    $finish;
  end


endmodule



module main
(
  input CLK,
  input RST
);

  reg [14-1:0] myram_0_addr;
  wire [32-1:0] myram_0_rdata;
  reg [32-1:0] myram_0_wdata;
  reg myram_0_wenable;
  reg [14-1:0] myram_1_addr;
  wire [32-1:0] myram_1_rdata;
  reg [32-1:0] myram_1_wdata;
  reg myram_1_wenable;

  myram
  inst_myram
  (
    .CLK(CLK),
    .myram_0_addr(myram_0_addr),
    .myram_0_rdata(myram_0_rdata),
    .myram_0_wdata(myram_0_wdata),
    .myram_0_wenable(myram_0_wenable),
    .myram_1_addr(myram_1_addr),
    .myram_1_rdata(myram_1_rdata),
    .myram_1_wdata(myram_1_wdata),
    .myram_1_wenable(myram_1_wenable)
  );

  reg [32-1:0] waddr;
  reg [32-1:0] count;
  reg _myram_cond_0_1;
  reg [32-1:0] raddr;
  reg [32-1:0] sum;
  localparam _tmp_0 = 1;
  wire [_tmp_0-1:0] _tmp_1;
  assign _tmp_1 = 1;
  reg [_tmp_0-1:0] __tmp_1_1;
  reg [_tmp_0-1:0] __tmp_1_2;
  reg [_tmp_0-1:0] __tmp_1_3;
  reg [_tmp_0-1:0] __tmp_1_4;
  reg _tmp_2;
  reg _myram_cond_1_1;
  reg _myram_cond_2_1;
  reg _myram_cond_2_2;
  reg _seq_cond_0_1;

  always @(posedge CLK) begin
    if(RST) begin
      waddr <= 0;
      count <= 0;
      __tmp_1_1 <= 0;
      __tmp_1_2 <= 0;
      __tmp_1_3 <= 0;
      __tmp_1_4 <= 0;
      raddr <= 0;
      sum <= 0;
      _seq_cond_0_1 <= 0;
    end else begin
      if(_seq_cond_0_1) begin
        $display("sum=%d", sum);
      end 
      if(waddr < 16) begin
        waddr <= waddr + 1;
        count <= count + 1;
      end 
      __tmp_1_1 <= _tmp_1;
      __tmp_1_2 <= __tmp_1_1;
      __tmp_1_3 <= __tmp_1_2;
      __tmp_1_4 <= __tmp_1_3;
      if(__tmp_1_4 && (raddr < 16)) begin
        raddr <= raddr + 1;
      end 
      if(_tmp_2) begin
        sum <= sum + myram_1_rdata;
      end 
      _seq_cond_0_1 <= _tmp_2;
    end
  end


  always @(posedge CLK) begin
    if(RST) begin
      myram_1_wdata <= 0;
      myram_1_wenable <= 0;
      myram_0_addr <= 0;
      myram_0_wdata <= 0;
      myram_0_wenable <= 0;
      _myram_cond_0_1 <= 0;
      myram_1_addr <= 0;
      _myram_cond_1_1 <= 0;
      _tmp_2 <= 0;
      _myram_cond_2_1 <= 0;
      _myram_cond_2_2 <= 0;
    end else begin
      if(_myram_cond_2_2) begin
        _tmp_2 <= 0;
      end 
      if(_myram_cond_0_1) begin
        myram_0_wenable <= 0;
      end 
      if(_myram_cond_1_1) begin
        _tmp_2 <= 1;
      end 
      _myram_cond_2_2 <= _myram_cond_2_1;
      myram_1_wdata <= 0;
      myram_1_wenable <= 0;
      if(waddr < 16) begin
        myram_0_addr <= waddr;
        myram_0_wdata <= count;
        myram_0_wenable <= 1;
      end 
      _myram_cond_0_1 <= waddr < 16;
      if(__tmp_1_4 && (raddr < 16)) begin
        myram_1_addr <= raddr;
      end 
      _myram_cond_1_1 <= __tmp_1_4 && (raddr < 16);
      _myram_cond_2_1 <= __tmp_1_4 && (raddr < 16);
    end
  end


endmodule



module myram
(
  input CLK,
  input [14-1:0] myram_0_addr,
  output [32-1:0] myram_0_rdata,
  input [32-1:0] myram_0_wdata,
  input myram_0_wenable,
  input [14-1:0] myram_1_addr,
  output [32-1:0] myram_1_rdata,
  input [32-1:0] myram_1_wdata,
  input myram_1_wenable
);

  reg [14-1:0] myram_0_daddr;
  reg [14-1:0] myram_1_daddr;
  reg [32-1:0] mem [0:16384-1];

  always @(posedge CLK) begin
    if(myram_0_wenable) begin
      mem[myram_0_addr] <= myram_0_wdata;
    end 
    myram_0_daddr <= myram_0_addr;
  end

  assign myram_0_rdata = mem[myram_0_daddr];

  always @(posedge CLK) begin
    if(myram_1_wenable) begin
      mem[myram_1_addr] <= myram_1_wdata;
    end 
    myram_1_daddr <= myram_1_addr;
  end

  assign myram_1_rdata = mem[myram_1_daddr];

endmodule
"""


def test():
    veriloggen.reset()
    test_module = types_ram_manager_seq.mkTest()
    code = test_module.to_verilog()

    from pyverilog.vparser.parser import VerilogParser
    from pyverilog.ast_code_generator.codegen import ASTCodeGenerator
    parser = VerilogParser()
    expected_ast = parser.parse(expected_verilog)
    codegen = ASTCodeGenerator()
    expected_code = codegen.visit(expected_ast)

    assert(expected_code == code)



from __future__ import absolute_import
from __future__ import print_function
import veriloggen
import types_fifo_delay

expected_verilog = """
module test;

  reg CLK;
  reg RST;

  main
  uut
  (
    .CLK(CLK),
    .RST(RST)
  );


  initial begin
    $dumpfile("uut.vcd");
    $dumpvars(0, uut, CLK, RST);
  end


  initial begin
    CLK = 0;
    forever begin
      #5 CLK = !CLK;
    end
  end


  initial begin
    RST = 0;
    #100;
    RST = 1;
    #100;
    RST = 0;
    #100000;
    $finish;
  end


endmodule



module main
(
  input CLK,
  input RST
);

  reg myfifo_enq;
  reg [32-1:0] myfifo_wdata;
  wire myfifo_full;
  wire myfifo_almost_full;
  reg myfifo_deq;
  wire [32-1:0] myfifo_rdata;
  wire myfifo_empty;
  wire myfifo_almost_empty;

  myfifo
  inst_myfifo
  (
    .CLK(CLK),
    .RST(RST),
    .myfifo_enq(myfifo_enq),
    .myfifo_wdata(myfifo_wdata),
    .myfifo_full(myfifo_full),
    .myfifo_almost_full(myfifo_almost_full),
    .myfifo_deq(myfifo_deq),
    .myfifo_rdata(myfifo_rdata),
    .myfifo_empty(myfifo_empty),
    .myfifo_almost_empty(myfifo_almost_empty)
  );

  reg [8-1:0] count_myfifo;
  reg [32-1:0] count;
  reg [32-1:0] sum;
  reg [32-1:0] fsm;
  localparam fsm_init = 0;
  reg [32-1:0] _myfifo_wdata_0_1;
  reg [32-1:0] _myfifo_wdata_0_2;
  reg [32-1:0] _myfifo_wdata_0_3;
  reg [32-1:0] _myfifo_wdata_0_4;
  reg _myfifo_cond_1_1;
  reg _myfifo_cond_1_2;
  reg _myfifo_cond_1_3;
  reg _myfifo_cond_1_4;
  reg _myfifo_cond_2_1;
  reg _myfifo_cond_2_2;
  reg _myfifo_cond_2_3;
  reg _myfifo_cond_2_4;
  reg _myfifo_cond_3_1;
  reg _myfifo_cond_3_2;
  reg _myfifo_cond_3_3;
  reg _myfifo_cond_3_4;
  reg _myfifo_cond_3_5;
  reg _tmp_0;
  reg _myfifo_cond_4_1;
  reg _myfifo_cond_5_1;
  reg _myfifo_cond_6_1;
  reg _myfifo_cond_6_2;
  reg [32-1:0] _d1_fsm;
  reg _fsm_cond_3_0_1;
  localparam fsm_1 = 1;
  localparam fsm_2 = 2;
  localparam fsm_3 = 3;
  localparam fsm_4 = 4;

  always @(posedge CLK) begin
    if(RST) begin
      fsm <= fsm_init;
      _d1_fsm <= fsm_init;
      count <= 0;
      sum <= 0;
      _fsm_cond_3_0_1 <= 0;
    end else begin
      _d1_fsm <= fsm;
      case(_d1_fsm)
        fsm_3: begin
          if(_fsm_cond_3_0_1) begin
            $display("sum=%d", sum);
          end 
        end
      endcase
      case(fsm)
        fsm_init: begin
          count <= 0;
          fsm <= fsm_1;
        end
        fsm_1: begin
          if(count_myfifo + 5 < 127) begin
            count <= count + 1;
          end 
          if(!myfifo_full && myfifo_enq) begin
            $display("count=%d space=%d has_space=%d", count_myfifo, (127 - count_myfifo), (count_myfifo + 1 < 127));
          end 
          if((count_myfifo + 5 < 127) && (count == 15)) begin
            fsm <= fsm_2;
          end 
        end
        fsm_2: begin
          count <= 0;
          fsm <= fsm_3;
        end
        fsm_3: begin
          if(_tmp_0) begin
            sum <= sum + myfifo_rdata;
            count <= count + 1;
            $write("count=%d space=%d has_space=%d ", count_myfifo, (127 - count_myfifo), (count_myfifo + 1 < 127));
          end 
          _fsm_cond_3_0_1 <= _tmp_0;
          if(count == 16) begin
            fsm <= fsm_4;
          end 
        end
      endcase
    end
  end


  always @(posedge CLK) begin
    if(RST) begin
      count_myfifo <= 0;
      _myfifo_wdata_0_1 <= 0;
      _myfifo_wdata_0_2 <= 0;
      _myfifo_wdata_0_3 <= 0;
      _myfifo_wdata_0_4 <= 0;
      _myfifo_cond_1_1 <= 0;
      _myfifo_cond_1_2 <= 0;
      _myfifo_cond_1_3 <= 0;
      _myfifo_cond_1_4 <= 0;
      myfifo_wdata <= 0;
      _myfifo_cond_2_1 <= 0;
      _myfifo_cond_2_2 <= 0;
      _myfifo_cond_2_3 <= 0;
      _myfifo_cond_2_4 <= 0;
      myfifo_enq <= 0;
      _myfifo_cond_3_1 <= 0;
      _myfifo_cond_3_2 <= 0;
      _myfifo_cond_3_3 <= 0;
      _myfifo_cond_3_4 <= 0;
      _myfifo_cond_3_5 <= 0;
      myfifo_deq <= 0;
      _myfifo_cond_4_1 <= 0;
      _tmp_0 <= 0;
      _myfifo_cond_5_1 <= 0;
      _myfifo_cond_6_1 <= 0;
      _myfifo_cond_6_2 <= 0;
    end else begin
      if(_myfifo_cond_3_5) begin
        myfifo_enq <= 0;
      end 
      if(_myfifo_cond_1_4) begin
        myfifo_wdata <= _myfifo_wdata_0_4;
      end 
      if(_myfifo_cond_2_4) begin
        myfifo_enq <= 1;
      end 
      _myfifo_cond_3_5 <= _myfifo_cond_3_4;
      _myfifo_wdata_0_4 <= _myfifo_wdata_0_3;
      _myfifo_cond_1_4 <= _myfifo_cond_1_3;
      _myfifo_cond_2_4 <= _myfifo_cond_2_3;
      _myfifo_cond_3_4 <= _myfifo_cond_3_3;
      _myfifo_wdata_0_3 <= _myfifo_wdata_0_2;
      _myfifo_cond_1_3 <= _myfifo_cond_1_2;
      _myfifo_cond_2_3 <= _myfifo_cond_2_2;
      _myfifo_cond_3_3 <= _myfifo_cond_3_2;
      if(_myfifo_cond_6_2) begin
        _tmp_0 <= 0;
      end 
      _myfifo_wdata_0_2 <= _myfifo_wdata_0_1;
      _myfifo_cond_1_2 <= _myfifo_cond_1_1;
      _myfifo_cond_2_2 <= _myfifo_cond_2_1;
      _myfifo_cond_3_2 <= _myfifo_cond_3_1;
      if(_myfifo_cond_4_1) begin
        _tmp_0 <= !myfifo_empty && myfifo_deq;
      end 
      if(_myfifo_cond_5_1) begin
        myfifo_deq <= 0;
      end 
      _myfifo_cond_6_2 <= _myfifo_cond_6_1;
      if(myfifo_enq && !myfifo_full && (myfifo_deq && !myfifo_empty)) begin
        count_myfifo <= count_myfifo;
      end else if(myfifo_enq && !myfifo_full) begin
        count_myfifo <= count_myfifo + 1;
      end else if(myfifo_deq && !myfifo_empty) begin
        count_myfifo <= count_myfifo - 1;
      end 
      _myfifo_wdata_0_1 <= count;
      _myfifo_cond_1_1 <= (fsm == 1) && !myfifo_full;
      _myfifo_cond_2_1 <= (fsm == 1) && !myfifo_full;
      _myfifo_cond_3_1 <= 1;
      if(fsm == 3) begin
        myfifo_deq <= 1;
      end 
      _myfifo_cond_4_1 <= fsm == 3;
      _myfifo_cond_5_1 <= 1;
      _myfifo_cond_6_1 <= 1;
    end
  end


endmodule



module myfifo
(
  input CLK,
  input RST,
  input myfifo_enq,
  input [32-1:0] myfifo_wdata,
  output myfifo_full,
  output myfifo_almost_full,
  input myfifo_deq,
  output [32-1:0] myfifo_rdata,
  output myfifo_empty,
  output myfifo_almost_empty
);

  reg [32-1:0] mem [0:128-1];
  reg [7-1:0] head;
  reg [7-1:0] tail;
  wire is_empty;
  wire is_almost_empty;
  wire is_full;
  wire is_almost_full;
  assign is_empty = head == tail;
  assign is_almost_empty = head == (tail + 1 & 127);
  assign is_full = (head + 1 & 127) == tail;
  assign is_almost_full = (head + 2 & 127) == tail;
  reg [32-1:0] rdata_reg;
  assign myfifo_full = is_full;
  assign myfifo_almost_full = is_almost_full || is_full;
  assign myfifo_empty = is_empty;
  assign myfifo_almost_empty = is_almost_empty || is_empty;
  assign myfifo_rdata = rdata_reg;

  always @(posedge CLK) begin
    if(RST) begin
      head <= 0;
      rdata_reg <= 0;
      tail <= 0;
    end else begin
      if(myfifo_enq && !is_full) begin
        mem[head] <= myfifo_wdata;
        head <= head + 1;
      end 
      if(myfifo_deq && !is_empty) begin
        rdata_reg <= mem[tail];
        tail <= tail + 1;
      end 
    end
  end


endmodule
"""

def test():
    veriloggen.reset()
    test_module = types_fifo_delay.mkTest()
    code = test_module.to_verilog()

    from pyverilog.vparser.parser import VerilogParser
    from pyverilog.ast_code_generator.codegen import ASTCodeGenerator
    parser = VerilogParser()
    expected_ast = parser.parse(expected_verilog)
    codegen = ASTCodeGenerator()
    expected_code = codegen.visit(expected_ast)

    assert(expected_code == code)



from __future__ import absolute_import
from __future__ import print_function
import veriloggen
import dataflow_getio


expected_verilog = """
module test;

  reg CLK;
  reg RST;

  main
  uut
  (
    .CLK(CLK),
    .RST(RST)
  );


  initial begin
    $dumpfile("uut.vcd");
    $dumpvars(0, uut, CLK, RST);
  end


  initial begin
    CLK = 0;
    forever begin
      #5 CLK = !CLK;
    end
  end


  initial begin
    RST = 0;
    #100;
    RST = 1;
    #100;
    RST = 0;
    #100000;
    $finish;
  end


endmodule



module main
(
  input CLK,
  input RST
);

  wire [32-1:0] xdata;
  wire xvalid;
  wire xready;
  wire [32-1:0] ydata;
  wire yvalid;
  wire yready;
  reg [32-1:0] _tmp_data_0;
  reg _tmp_valid_0;
  wire _tmp_ready_0;
  assign xready = (_tmp_ready_0 || !_tmp_valid_0) && (xvalid && yvalid);
  assign yready = (_tmp_ready_0 || !_tmp_valid_0) && (xvalid && yvalid);
  wire [32-1:0] zdata;
  wire zvalid;
  wire zready;
  assign zdata = _tmp_data_0;
  assign zvalid = _tmp_valid_0;
  assign _tmp_ready_0 = zready;
  reg [32-1:0] xfsm;
  localparam xfsm_init = 0;
  reg [32-1:0] _tmp_1;
  reg [32-1:0] _tmp_2;
  assign xdata = _tmp_2;
  reg _tmp_3;
  assign xvalid = _tmp_3;
  reg __dataflow_seq_0_cond_0_1;
  localparam xfsm_1 = 1;

  always @(posedge CLK) begin
    if(RST) begin
      xfsm <= xfsm_init;
      _tmp_1 <= 0;
    end else begin
      case(xfsm)
        xfsm_init: begin
          if(xready || !_tmp_3) begin
            _tmp_1 <= _tmp_1 + 1;
          end 
          if((xready || !_tmp_3) && (_tmp_1 == 15)) begin
            xfsm <= xfsm_1;
          end 
        end
      endcase
    end
  end

  reg [32-1:0] yfsm;
  localparam yfsm_init = 0;
  reg [32-1:0] _tmp_4;
  reg [32-1:0] _tmp_5;
  assign ydata = _tmp_5;
  reg _tmp_6;
  assign yvalid = _tmp_6;
  reg __dataflow_seq_0_cond_1_1;
  localparam yfsm_1 = 1;

  always @(posedge CLK) begin
    if(RST) begin
      yfsm <= yfsm_init;
      _tmp_4 <= 0;
    end else begin
      case(yfsm)
        yfsm_init: begin
          if(yready || !_tmp_6) begin
            _tmp_4 <= _tmp_4 + 1;
          end 
          if((yready || !_tmp_6) && (_tmp_4 == 15)) begin
            yfsm <= yfsm_1;
          end 
        end
      endcase
    end
  end

  assign zready = 1;

  always @(posedge CLK) begin
    if(zvalid) begin
      $display("zdata=%d", zdata);
    end 
  end


  always @(posedge CLK) begin
    if(RST) begin
      _tmp_data_0 <= 0;
      _tmp_valid_0 <= 0;
      _tmp_2 <= 0;
      _tmp_3 <= 0;
      __dataflow_seq_0_cond_0_1 <= 0;
      _tmp_5 <= 0;
      _tmp_6 <= 0;
      __dataflow_seq_0_cond_1_1 <= 0;
    end else begin
      if(__dataflow_seq_0_cond_0_1) begin
        _tmp_3 <= 0;
      end 
      if(__dataflow_seq_0_cond_1_1) begin
        _tmp_6 <= 0;
      end 
      if((_tmp_ready_0 || !_tmp_valid_0) && (xready && yready) && (xvalid && yvalid)) begin
        _tmp_data_0 <= xdata + ydata;
      end 
      if(_tmp_valid_0 && _tmp_ready_0) begin
        _tmp_valid_0 <= 0;
      end 
      if((_tmp_ready_0 || !_tmp_valid_0) && (xready && yready)) begin
        _tmp_valid_0 <= xvalid && yvalid;
      end 
      if((xfsm == 0) && (xready || !_tmp_3)) begin
        _tmp_2 <= _tmp_1;
      end 
      if((xfsm == 0) && (xready || !_tmp_3)) begin
        _tmp_3 <= 1;
      end 
      __dataflow_seq_0_cond_0_1 <= 1;
      if(_tmp_3 && !xready) begin
        _tmp_3 <= _tmp_3;
      end 
      if((yfsm == 0) && (yready || !_tmp_6)) begin
        _tmp_5 <= _tmp_4;
      end 
      if((yfsm == 0) && (yready || !_tmp_6)) begin
        _tmp_6 <= 1;
      end 
      __dataflow_seq_0_cond_1_1 <= 1;
      if(_tmp_6 && !yready) begin
        _tmp_6 <= _tmp_6;
      end 
    end
  end


endmodule
"""

def test():
    veriloggen.reset()
    test_module = dataflow_getio.mkTest()
    code = test_module.to_verilog()

    from pyverilog.vparser.parser import VerilogParser
    from pyverilog.ast_code_generator.codegen import ASTCodeGenerator
    parser = VerilogParser()
    expected_ast = parser.parse(expected_verilog)
    codegen = ASTCodeGenerator()
    expected_code = codegen.visit(expected_ast)

    assert(expected_code == code)



from __future__ import absolute_import
from __future__ import print_function
import sys
import os

# the next line can be removed after installation
sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))))

from veriloggen import *

# external function definition
def get_ext():
    ext = Function('ext_inc', 32)
    ext_v = ext.Input('v', 32)
    ext_o = ext.Input('o', 32)
    ext_tmp = ext.Reg('tmp', 32)
    ext.Body(
        ext_tmp(ext_v + ext_o),
        ext(ext_tmp)
    )
    return ext
    
def mkLed():
    m = Module('blinkled')
    width = m.Parameter('WIDTH', 8)
    clk = m.Input('CLK')
    rst = m.Input('RST')
    led = m.OutputReg('LED', width)
    count = m.Reg('count', 32)

    inc = m.Function('inc', width)
    inc_v = inc.Input('v', width)
    inc_o = inc.Input('o', width)
    inc_tmp = inc.Reg('tmp', width)
    inc.Body(
        inc_tmp(inc_v + inc_o),
        inc(inc_tmp)
    )

    # import an external function definition
    m.add_function(get_ext())
    
    m.Always(Posedge(clk))(
        If(rst)(
            count(0)
        ).Else(
            If(count == 1023)(
                count(0)
            ).Else(
                #count( FunctionCall(inc, count, 1) + 1 - 1 )
                count( inc.call(count, 1) + 1 - 1 )
            )
        ))
    
    m.Always(Posedge(clk))(
        If(rst)(
            led(1)
        ).Else(
            If(count == 1024 - 1)(
                led[0](led[width-1]),
                led[1:width](led[0:width-1])
            )
        ))

    return m

if __name__ == '__main__':
    led = mkLed()
    verilog = led.to_verilog()
    print(verilog)



from __future__ import absolute_import
from __future__ import print_function
import sys
import os

# the next line can be removed after installation
sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.dirname(
    os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))))))

from veriloggen import *
import veriloggen.types.axi as axi


def mkMain():
    m = Module('main')
    clk = m.Input('CLK')
    rst = m.Input('RST')

    myaxi = axi.AxiMaster(m, 'myaxi', clk, rst)
    myaxi.disable_write()

    fsm = FSM(m, 'fsm', clk, rst)
    sum = m.Reg('sum', 32, initval=0)

    # read address
    araddr = 1024
    arlen = 64
    expected_sum = (araddr + araddr + arlen - 1) * arlen // 2

    ack, counter = myaxi.read_request(araddr, arlen, cond=fsm)
    fsm.If(ack).goto_next()

    # read data
    ready = m.Wire('ready')
    ready.assign(myaxi.rdata.rvalid)
    data, valid, last = myaxi.read_data(counter, cond=(fsm, ready))

    fsm.If(Ands(valid, ready))(
        sum(sum + data)
    )
    fsm.Then().If(last).goto_next()

    fsm(
        Systask('display', 'sum=%d expected_sum=%d', sum, expected_sum)
    )
    fsm.goto_next()

    fsm.make_always()

    return m


def mkTest():
    m = Module('test')

    # target instance
    main = mkMain()

    # copy paras and ports
    params = m.copy_params(main)
    ports = m.copy_sim_ports(main)

    clk = ports['CLK']
    rst = ports['RST']

    # awready (no stall)
    awready = ports['myaxi_awready']
    _awready = m.TmpWireLike(awready)
    _awready.assign(1)
    m.Always()(awready(_awready))

    # wready (nostall)
    wready = ports['myaxi_wready']
    _wready = m.TmpWireLike(wready)
    _wready.assign(1)
    m.Always()(wready(_wready))

    # arready (no stall)
    #arready = ports['myaxi_arready']
    #_arready = m.TmpWireLike(arready)
    #_arready.assign(0)
    #m.Always()( arready(_arready) )

    # arready, rvalid, rdata, rlast
    raddr_fsm = FSM(m, 'raddr', clk, rst)
    _arlen = m.Reg('_arlen', 32, initval=0)
    raddr_fsm(
        ports['myaxi_arready'](0),
        ports['myaxi_rdata'](-1),
        ports['myaxi_rvalid'](0),
        ports['myaxi_rlast'](0)
    )
    raddr_fsm.If(ports['myaxi_arvalid']).goto_next()

    raddr_fsm.If(ports['myaxi_arvalid'])(
        ports['myaxi_arready'](1),
        ports['myaxi_rdata'](ports['myaxi_araddr'] - 1)
    )
    raddr_fsm.goto_next()

    raddr_fsm(
        ports['myaxi_arready'](0),
        _arlen(ports['myaxi_arlen'])
    )
    raddr_fsm.goto_next()

    ack = Ors(ports['myaxi_rready'], Not(ports['myaxi_rvalid']))

    raddr_fsm.If(Ands(ack, Not(ports['myaxi_rlast'])))(
        ports['myaxi_rdata'].inc(),
        ports['myaxi_rvalid'](1),
        ports['myaxi_rlast'](0),
        _arlen.dec()
    )
    raddr_fsm.Then().If(_arlen == 0)(
        ports['myaxi_rlast'](1),
    )
    raddr_fsm.Delay(1)(
        ports['myaxi_rvalid'](0),
        ports['myaxi_rlast'](0)
    )
    raddr_fsm.If(Ands(ports['myaxi_rvalid'], Not(ports['myaxi_rready'])))(
        ports['myaxi_rvalid'](ports['myaxi_rvalid']),
        ports['myaxi_rlast'](ports['myaxi_rlast']),
    )
    raddr_fsm.If(Ands(ports['myaxi_rvalid'],
                      ports['myaxi_rready'], ports['myaxi_rlast'])).goto_next()

    raddr_fsm.goto_next()

    raddr_fsm.goto_init()

    raddr_fsm.make_always()

    uut = m.Instance(main, 'uut',
                     params=m.connect_params(main),
                     ports=m.connect_ports(main))

    simulation.setup_waveform(m, uut, m.get_vars())
    simulation.setup_clock(m, clk, hperiod=5)
    init = simulation.setup_reset(m, rst, m.make_reset(), period=100)

    init.add(
        Delay(1000 * 100),
        Systask('finish'),
    )

    return m

if __name__ == '__main__':
    test = mkTest()
    verilog = test.to_verilog('tmp.v')
    print(verilog)

    sim = simulation.Simulator(test)
    rslt = sim.run()
    print(rslt)

    # sim.view_waveform()



from __future__ import absolute_import
from __future__ import print_function
import sys
import os
import math

# the next line can be removed after installation
sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__)))))))

from veriloggen import *

def mkLed(window_size=8):
    if window_size < 0:
        raise ValueError('window_size must be larger than 0')
    if math.log(window_size, 2) % 1.0 != 0.0:
        raise ValueError('window_size must be power of 2')
        
    m = Module('blinkled')
    width = m.Parameter('WIDTH', 32)
    clk = m.Input('CLK')
    rst = m.Input('RST')
    x = m.Input('x', width)
    y = m.OutputReg('y', width, initval=0)
    
    seq = Seq(m, 'seq', clk, rst)
    x = x + 100
    v = x
    for w in range(window_size-1):
        v = v + seq.Prev(x, w + 1)
        
    t = m.TmpReg(32)
    seq(
        t(v)
    )
    seq(
        y(t >> int(math.log(window_size, 2)))
    )
    
    seq.make_always()

    return m

def mkTest():
    m = Module('test')

    # target instance
    led = mkLed()
    
    # copy paras and ports
    params = m.copy_params(led)
    ports = m.copy_sim_ports(led)

    clk = ports['CLK']
    rst = ports['RST']
    x = ports['x']
    y = ports['y']
    
    uut = m.Instance(led, 'uut',
                     params=m.connect_params(led),
                     ports=m.connect_ports(led))

    reset_stmt = []
    reset_stmt.append( x(0) )
    
    simulation.setup_waveform(m, uut)
    simulation.setup_clock(m, clk, hperiod=5)
    init = simulation.setup_reset(m, rst, reset_stmt, period=100)

    nclk = simulation.next_clock
    
    init.add(
        Delay(1000),
        
        [ (nclk(clk), x(i+10)) for i in range(10) ],
        nclk(clk), x(0),
        
        Delay(1000),
        Systask('finish'),
    )

    return m
    
if __name__ == '__main__':
    test = mkTest()
    verilog = test.to_verilog('tmp.v')
    print(verilog)



from __future__ import absolute_import
from __future__ import print_function
import sys
import os
import math

# the next line can be removed after installation
sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__)))))))

from veriloggen import *
import veriloggen.dataflow as dataflow

def mkMain(n=128, datawidth=32, numports=2):
    m = Module('main')

    clk = m.Input('CLK')
    rst = m.Input('RST')

    # counters
    a = dataflow.Counter(maxval=8, width=8)
    b = dataflow.Counter(step=2, maxval=16, width=8)
    
    c = a + b
    c.output('cdata', valid='cvalid')

    # synthesize dataflow
    df = dataflow.Dataflow(c)
    df.implement(m, clk, rst, aswire=False)
    #df.draw_graph()
    
    fsm = FSM(m, 'fsm', clk, rst)
    count = m.TmpReg(32, initval=0)

    fsm.If(c.valid)(
        count.inc(),
        Systask('display', "c=%d", c.data)
    )

    fsm.If(count == 32).goto_next()
    
    fsm.make_always()
    
    return m
    
def mkTest():
    m = Module('test')
    
    # target instance
    main = mkMain()
    
    # copy paras and ports
    params = m.copy_params(main)
    ports = m.copy_sim_ports(main)
    
    clk = ports['CLK']
    rst = ports['RST']
    
    uut = m.Instance(main, 'uut',
                     params=m.connect_params(main),
                     ports=m.connect_ports(main))
    
    simulation.setup_waveform(m, uut, m.get_vars())
    simulation.setup_clock(m, clk, hperiod=5)
    init = simulation.setup_reset(m, rst, m.make_reset(), period=100)
    
    init.add(
        Delay(1000 * 100),
        Systask('finish'),
    )

    return m
    
if __name__ == '__main__':
    test = mkTest()
    verilog = test.to_verilog('tmp.v')
    print(verilog)

    sim = simulation.Simulator(test)
    rslt = sim.run()
    print(rslt)

    #sim.view_waveform()



from __future__ import absolute_import
from __future__ import print_function
import veriloggen
import pipeline_acc_add_validready

expected_verilog = """
module test;
  reg CLK;
  reg RST;
  reg [32-1:0] x;
  reg vx;
  wire rx;
  wire [32-1:0] y;
  wire vy;
  reg ry;
  reg prst;

  blinkled
  uut
  (
    .CLK(CLK),
    .RST(RST),
    .x(x),
    .vx(vx),
    .rx(rx),
    .y(y),
    .vy(vy),
    .ry(ry),
    .prst(prst)
  );

  reg reset_done;

  initial begin
    $dumpfile("uut.vcd");
    $dumpvars(0, uut);
  end


  initial begin
    CLK = 0;
    forever begin
      #5 CLK = !CLK;
    end
  end


  initial begin
    RST = 0;
    reset_done = 0;
    prst = 0;
    x = 0;
    vx = 0;
    ry = 0;
    #100;
    RST = 1;
    #100;
    RST = 0;
    #1000;
    reset_done = 1;
    @(posedge CLK);
    #1;
    #10000;
    $finish;
  end

  reg [32-1:0] _tmp_0;
  reg [32-1:0] _tmp_1;
  reg [32-1:0] xfsm;
  localparam xfsm_init = 0;
  localparam xfsm_1 = 1;
  localparam xfsm_2 = 2;
  localparam xfsm_3 = 3;
  localparam xfsm_4 = 4;
  localparam xfsm_5 = 5;
  localparam xfsm_6 = 6;
  localparam xfsm_7 = 7;
  localparam xfsm_8 = 8;
  localparam xfsm_9 = 9;
  localparam xfsm_10 = 10;
  localparam xfsm_11 = 11;
  localparam xfsm_12 = 12;
  localparam xfsm_13 = 13;
  localparam xfsm_14 = 14;
  localparam xfsm_15 = 15;
  localparam xfsm_16 = 16;
  localparam xfsm_17 = 17;
  localparam xfsm_18 = 18;
  localparam xfsm_19 = 19;
  localparam xfsm_20 = 20;
  localparam xfsm_21 = 21;
  localparam xfsm_22 = 22;
  localparam xfsm_23 = 23;
  localparam xfsm_24 = 24;

  always @(posedge CLK) begin
    if(RST) begin
      xfsm <= xfsm_init;
      _tmp_0 <= 0;
    end else begin
      case(xfsm)
        xfsm_init: begin
          vx <= 0;
          if(reset_done) begin
            xfsm <= xfsm_1;
          end 
        end
        xfsm_1: begin
          xfsm <= xfsm_2;
        end
        xfsm_2: begin
          xfsm <= xfsm_3;
        end
        xfsm_3: begin
          xfsm <= xfsm_4;
        end
        xfsm_4: begin
          xfsm <= xfsm_5;
        end
        xfsm_5: begin
          xfsm <= xfsm_6;
        end
        xfsm_6: begin
          xfsm <= xfsm_7;
        end
        xfsm_7: begin
          xfsm <= xfsm_8;
        end
        xfsm_8: begin
          xfsm <= xfsm_9;
        end
        xfsm_9: begin
          xfsm <= xfsm_10;
        end
        xfsm_10: begin
          xfsm <= xfsm_11;
        end
        xfsm_11: begin
          vx <= 1;
          xfsm <= xfsm_12;
        end
        xfsm_12: begin
          if(rx) begin
            x <= x + 1;
          end 
          if(rx) begin
            _tmp_0 <= _tmp_0 + 1;
          end 
          if((_tmp_0 == 5) && rx) begin
            xfsm <= xfsm_13;
          end 
        end
        xfsm_13: begin
          vx <= 0;
          xfsm <= xfsm_14;
        end
        xfsm_14: begin
          xfsm <= xfsm_15;
        end
        xfsm_15: begin
          xfsm <= xfsm_16;
        end
        xfsm_16: begin
          xfsm <= xfsm_17;
        end
        xfsm_17: begin
          xfsm <= xfsm_18;
        end
        xfsm_18: begin
          xfsm <= xfsm_19;
        end
        xfsm_19: begin
          xfsm <= xfsm_20;
        end
        xfsm_20: begin
          xfsm <= xfsm_21;
        end
        xfsm_21: begin
          xfsm <= xfsm_22;
        end
        xfsm_22: begin
          xfsm <= xfsm_23;
        end
        xfsm_23: begin
          vx <= 1;
          if(rx) begin
            x <= x + 1;
          end 
          if(rx) begin
            _tmp_0 <= _tmp_0 + 1;
          end 
          if((_tmp_0 == 10) && rx) begin
            xfsm <= xfsm_24;
          end 
        end
        xfsm_24: begin
          vx <= 0;
        end
      endcase
    end
  end

  reg [32-1:0] yfsm;
  localparam yfsm_init = 0;
  localparam yfsm_1 = 1;
  localparam yfsm_2 = 2;
  localparam yfsm_3 = 3;
  localparam yfsm_4 = 4;
  localparam yfsm_5 = 5;
  localparam yfsm_6 = 6;
  localparam yfsm_7 = 7;
  localparam yfsm_8 = 8;
  localparam yfsm_9 = 9;
  localparam yfsm_10 = 10;
  localparam yfsm_11 = 11;
  localparam yfsm_12 = 12;
  localparam yfsm_13 = 13;

  always @(posedge CLK) begin
    if(RST) begin
      yfsm <= yfsm_init;
    end else begin
      case(yfsm)
        yfsm_init: begin
          ry <= 0;
          if(reset_done) begin
            yfsm <= yfsm_1;
          end 
        end
        yfsm_1: begin
          yfsm <= yfsm_2;
        end
        yfsm_2: begin
          if(vy) begin
            ry <= 1;
          end 
          if(vy) begin
            yfsm <= yfsm_3;
          end 
        end
        yfsm_3: begin
          ry <= 0;
          yfsm <= yfsm_4;
        end
        yfsm_4: begin
          ry <= 0;
          yfsm <= yfsm_5;
        end
        yfsm_5: begin
          ry <= 0;
          yfsm <= yfsm_6;
        end
        yfsm_6: begin
          ry <= 0;
          yfsm <= yfsm_7;
        end
        yfsm_7: begin
          ry <= 0;
          yfsm <= yfsm_8;
        end
        yfsm_8: begin
          ry <= 0;
          yfsm <= yfsm_9;
        end
        yfsm_9: begin
          ry <= 0;
          yfsm <= yfsm_10;
        end
        yfsm_10: begin
          ry <= 0;
          yfsm <= yfsm_11;
        end
        yfsm_11: begin
          ry <= 0;
          yfsm <= yfsm_12;
        end
        yfsm_12: begin
          ry <= 0;
          yfsm <= yfsm_13;
        end
        yfsm_13: begin
          yfsm <= yfsm_2;
        end
      endcase
    end
  end


  always @(posedge CLK) begin
    if(reset_done) begin
      if(vx && rx) begin
        $display("x=%d", x);
      end 
      if(vy && ry) begin
        $display("y=%d", y);
      end 
    end 
  end


endmodule


module blinkled
(
  input CLK,
  input RST,
  input [32-1:0] x,
  input vx,
  output rx,
  output [32-1:0] y,
  output vy,
  input ry,
  input prst
);

  assign rx = (_df_ready_0 || !_df_valid_0) && vx;
  reg [32-1:0] _df_data_0;
  reg _df_valid_0;
  wire _df_ready_0;
  assign _df_ready_0 = ry;
  assign y = _df_data_0;
  assign vy = _df_valid_0;

  always @(posedge CLK) begin
    if(RST) begin
      _df_data_0 <= 0;
      _df_valid_0 <= 0;
    end else begin
      if(vx && rx && (_df_ready_0 || !_df_valid_0)) begin
        _df_data_0 <= _df_data_0 + x;
      end 
      if(_df_valid_0 && _df_ready_0) begin
        _df_valid_0 <= 0;
      end 
      if(rx && (_df_ready_0 || !_df_valid_0)) begin
        _df_valid_0 <= vx;
      end 
      if(prst) begin
        _df_data_0 <= 0;
      end 
      if(prst) begin
        _df_valid_0 <= 0;
      end 
    end
  end

endmodule
"""

def test():
    veriloggen.reset()
    test_module = pipeline_acc_add_validready.mkTest()
    code = test_module.to_verilog()

    from pyverilog.vparser.parser import VerilogParser
    from pyverilog.ast_code_generator.codegen import ASTCodeGenerator
    parser = VerilogParser()
    expected_ast = parser.parse(expected_verilog)
    codegen = ASTCodeGenerator()
    expected_code = codegen.visit(expected_ast)

    assert(expected_code == code)



from __future__ import absolute_import
from __future__ import print_function
import sys
import os

# the next line can be removed after installation
sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__)))))))

from veriloggen import *
import veriloggen.dataflow as dataflow

def mkMain():
    # input variiable
    x = dataflow.Variable('xdata', valid='xvalid', ready='xready', signed=True)
    y = dataflow.Variable('ydata', valid='yvalid', ready='yready', signed=True)

    # dataflow definition
    z = x * y

    # set output attribute
    z.output('zdata', valid='zvalid', ready='zready')

    df = dataflow.Dataflow(z)
    m = df.to_module('main')
    
    return m

def mkTest(numports=8):
    m = Module('test')

    # target instance
    main = mkMain()

    params = m.copy_params(main)
    ports = m.copy_sim_ports(main)

    clk = ports['CLK']
    rst = ports['RST']

    xdata = ports['xdata']
    xvalid = ports['xvalid']
    xready = ports['xready']
    
    ydata = ports['ydata']
    yvalid = ports['yvalid']
    yready = ports['yready']
    
    zdata = ports['zdata']
    zvalid = ports['zvalid']
    zready = ports['zready']
    
    uut = m.Instance(main, 'uut',
                     params=m.connect_params(main),
                     ports=m.connect_ports(main))

    reset_done = m.Reg('reset_done', initval=0)
    reset_stmt = []
    reset_stmt.append( reset_done(0) )
    reset_stmt.append( xdata(0) )
    reset_stmt.append( xvalid(0) )
    reset_stmt.append( ydata(0) )
    reset_stmt.append( yvalid(0) )
    reset_stmt.append( zready(0) )

    simulation.setup_waveform(m, uut)
    simulation.setup_clock(m, clk, hperiod=5)
    init = simulation.setup_reset(m, rst, reset_stmt, period=100)

    nclk = simulation.next_clock
    
    init.add(
        Delay(1000),
        reset_done(1),
        nclk(clk),
        Delay(10000),
        Systask('finish'),
    )

    def send(name, data, valid, ready, step=1, waitnum=10):
        fsm = FSM(m, name + 'fsm', clk, rst)
        count = m.TmpReg(32, initval=0)
        
        fsm.add(valid(0))
        fsm.goto_next(cond=reset_done)
        for _ in range(waitnum):
            fsm.goto_next()
            
        fsm.add(valid(1))
        fsm.goto_next()
        
        fsm.add(data(data + step), cond=ready)
        fsm.add(count.inc(), cond=ready)
        fsm.add(valid(0), cond=AndList(count==5, ready))
        fsm.goto_next(cond=AndList(count==5, ready))
        
        for _ in range(waitnum):
            fsm.goto_next()
        fsm.add(valid(1))
        
        fsm.add(data(data + step), cond=ready)
        fsm.add(count.inc(), cond=ready)
        fsm.add(valid(0), cond=AndList(count==10, ready))
        fsm.goto_next(cond=AndList(count==10, ready))
        
        fsm.make_always()
    

    def receive(name, data, valid, ready, waitnum=10):
        fsm = FSM(m, name + 'fsm', clk, rst)
        
        fsm.add(ready(0))
        fsm.goto_next(cond=reset_done)
        fsm.goto_next()
        
        yinit= fsm.current
        fsm.add(ready(1), cond=valid)
        fsm.goto_next(cond=valid)
        for i in range(waitnum):
            fsm.add(ready(0))
            fsm.goto_next()
            
        fsm.goto(yinit)
        
        fsm.make_always()

        
    send('x', xdata, xvalid, xready, step=-1, waitnum=10)
    send('y', ydata, yvalid, yready, step=2, waitnum=20)
    receive('z', zdata, zvalid, zready, waitnum=50)
    
    
    m.Always(Posedge(clk))(
        If(reset_done)(
            If(AndList(xvalid, xready))(
                Systask('display', 'xdata=%d', xdata)
            ),
            If(AndList(yvalid, yready))(
                Systask('display', 'ydata=%d', ydata)
            ),
            If(AndList(zvalid, zready))(
                Systask('display', 'zdata=%d', zdata)
            )
        )
    )
    
    return m

    
if __name__ == '__main__':
    test = mkTest()
    verilog = test.to_verilog('tmp.v')
    print(verilog)

    # run simulator (Icarus Verilog)
    sim = simulation.Simulator(test)
    rslt = sim.run() # display=False
    #rslt = sim.run(display=True)
    print(rslt)

    # launch waveform viewer (GTKwave)
    #sim.view_waveform() # background=False
    #sim.view_waveform(background=True)
